import json
import logging
import sys
from argparse import ArgumentParser

import dtiot_d2c.d2c as d2c
import dtiot_d2c.dmo as dmo
from dtiot_d2c.cli.cli_command import ApiConnCmdLineInterface
from dtiot_d2c.d2c import CMDARGS, Device, DeviceGroup, OneM2MBase
from dtiot_d2c.d2c import utils as utils
from dtiot_d2c.d2c.get_response import GetResponse
from dtiot_d2c.d2c.utils import color
from dtiot_d2c.dmo import dmoCLI as dmocli

log = logging.getLogger(__name__)
            
def print_flushed(s:str, end=None):
    print(s, end=end)
    sys.stdout.flush()    

def print_red(s:str, end=None):
    print_flushed(f"{color.RED}{s}{color.END}", end=end)

def print_green(s:str, end=None):
    print_flushed(f"{color.GREEN}{s}{color.END}", end=end)

def check_if_ae_exists(api_conn, origin, rn:str)->bool:
    print_flushed(f"Checking ae  '{rn}' ... ", end="")
    
    response = dmo.get_resources(api_conn, origin, resourceName=rn)
    if not response:
        print_red(f"NOT OK. Ae {rn} not found!")  
        return False      
    else:
        print_green("OK")
        return True

def check_if_cnt_exists(api_conn, origin, rn:str)->bool:
    print_flushed(f"Checking cnt '{rn}' ... ", end="")
    response = dmo.get_resources(api_conn, origin, resourceName=rn)
    if not response:
        print_red(f"NOT OK. Cnt {rn} not found!")  
        return False      
    else:
        print_green("OK")
        return True
    
def check_if_sub_exists(api_conn, origin, parent_rn:str, sub_rn:str, url:str=None)->bool:
    if parent_rn:
        print_flushed(f"Checking sub '{sub_rn}' in '{parent_rn}' ... ", end="")
        response = dmo.get_resources(api_conn, origin, resourceName=parent_rn)
    else:
        print_flushed(f"Checking sub '{sub_rn}' ... ", end="")
        response = dmo.get_resources(api_conn, origin, resourceType="sub")

    if not response:
        print_red(f"NOT OK. Resource {parent_rn} not found!")  
        return False      

    subs = []    
    if parent_rn:
        subs = response.get("m2m:sub", [])
        if not subs:
            print_red(f"NOT OK. No subscriptions in {parent_rn} found!")  
            return False    
    else:
        subs = response
        
    #print(json.dumps(subs))

    found_sub = None
    for sub in subs:
        rn = sub.get("rn", None)
        if rn and rn == sub_rn:
            found_sub = sub
            break

    if not found_sub:
        print_red(f"NOT OK. No subscription {sub_rn} not found in {parent_rn}!")  
        return False      

    if url:
        if not (urls := found_sub.get("nu", [])):
            print_red(f"NOT OK. No url defined in subscription {sub_rn}!")
            return False

        if url != urls[0]:
            print_red(f"WARNING. Urls of subscription {sub_rn} are different!")
            print_red(f"   Check url: {url}")
            print_red(f"   Found url: {urls[0]}")
            return False      
        else:     
            print_green("OK")
            print_flushed(f"   Check url: {url}")
            print_flushed(f"   Found url: {urls[0]}")
            return True
    else:
        print_green("OK")
        return True
    
def check_if_cnt_in_ae_exists(api_conn, origin, ae_rn:str, cnt_rn:str)->bool:
    print_flushed(f"Checking cnt '{cnt_rn}' exists in ae '{ae_rn}' ... ", end="")

    if not (response := dmo.get_resources(api_conn, origin, resourceName=ae_rn)):
        print_red(f"NOT OK. Ae {ae_rn} not found!")  
        return False      

    if not (cnts := response.get("m2m:cnt", None)):
        print_red(f"NOT OK. No containers in AE {ae_rn}!")  
        return False      

    found = False
    for cnt in cnts:
        rn = cnt.get("rn", None)
        if rn and rn == cnt_rn:
            found = True
            break

    if not found:
        print_red(f"NOT OK. No container {cnt_rn} not found in AE {ae_rn}!")  
        return False      
    else:
        print_green("OK")
        return True
            
#####################################################################################                
class VerifyD2CModelCLI(ApiConnCmdLineInterface):

    def __init__(self):
        super().__init__()

    def get1LineHelp(self):
        return "Creates a new device group."
    
    def addCmdLineArgs(self, argsParser):
        self.addCmdLineArgsFromTemplate(CMDARGS, argsParser, "-o")

    def main(self, cmdargs=None):

        ### NI-IPE
        # Check if AE NI-IPE 
        niipe_exists = check_if_ae_exists(self.apiConn, cmdargs.origin, "NI-IPE")
                    
        ### device-provisioning
        devprov_exists = check_if_ae_exists(self.apiConn, cmdargs.origin, "device-provisioning")        
        devprov_sub_exists = check_if_sub_exists(self.apiConn, cmdargs.origin, None, "subscription",
                                                 url="http://d2c-device-provisioning.d2c.svc.cluster.local:80/provisioning/application-entity")
        request_exists = check_if_cnt_in_ae_exists(self.apiConn, cmdargs.origin, "device-provisioning", "request")
        response_exists = check_if_cnt_in_ae_exists(self.apiConn, cmdargs.origin, "device-provisioning", "response")
        request_sub_exists = check_if_sub_exists(self.apiConn, cmdargs.origin, "device-provisioning/request", "subscription",
                                                 url="http://d2c-device-provisioning.d2c.svc.cluster.local/provisioning/node-list-provisioning")

            
        ### device-group
        # Check if AE device-group
        devgrp_exists = check_if_ae_exists(self.apiConn, cmdargs.origin, "device-group")     
        devgrp_sub_exists = check_if_sub_exists(self.apiConn, cmdargs.origin, "device-group", "subscription",
                                                url="http://d2c-device-provisioning.d2c.svc.cluster.local/provisioning/device-group")
        
        ### device-communication
        devcomm_exists = check_if_cnt_exists(self.apiConn, cmdargs.origin, "device-communication")

            



from enum import Enum

#OneM2m Dateformat
ONEM2M_DATE_FORMAT = "%Y%m%dT%H%M%S"

# DMO basepath
DMO_BASEPATH="api-gw/device-management-orchestrator"

# OneM2M resource type ids
TYs = {
    'acp':1,             
    'ae':2,                     # AE
    'application-entity':2,
    'cnt':3,                    # CONTAINER
    'container':3,
    'cin':4,                    # CONTENT INSTANCE
    'content-instance':4,
    'cb':5,                     # CSEBASE
    'pc':5,                     # POLLING CHANNEL        
    'sch':7,                    # SCHEDULE
    'grp':9,                    # GROUP 
    'group':9,
    'mob':13,                   # Managed Object
    'managed-object':13,
    'nod':14,                   # NODE
    'node':14,
    'csr':16,                   # REMOTECSE
    'sub':23,                   # SUBSCRIPTIONS
    'subscription':23,
    'smd':27,                   # SEMANTIC DESCRIPTOR
    'fcn':28,                   # FLEX CONTAINER 
    'flex-container':28         
}

TYs_JSON_FILTERS = {
    'acp':None,             
    'ae':"m2m:cb.m2m:ae",                      # AE
    'application-entity':"m2m:cb.m2m:ae",
    'cnt':"m2m:cb.m2m:cnt",                    # CONTAINER
    'container':"m2m:cb.m2m:cnt",
    'cin':None,                                # CONTENT INSTANCE
    'content-instance':None,                      
    'cb':"m2m:cb",                             # CSEBASE
    'pc':None,                                 # POLLING CHANNEL        
    'sch':None,                                # SCHEDULE
    'grp':None,                                # GROUP 
    'group':None,
    'mob':None,                                # Managed Object
    'managed-object':None,

    'nod':"m2m:cb.m2m:nod",                    # NODE
    'node':"m2m:cb.m2m:nod",

    'csr':None,                                # REMOTECSE
    'sub':"m2m:cb.m2m:sub",                    # SUBSCRIPTIONS
    'subscription':"m2m:cb.m2m:sub",                         
    'smd':None,                                # SEMANTIC DESCRIPTOR
    'fcn':"m2m:cb.m2m:flexContainer",          # FLEX CONTAINER 
    'flex-container':"m2m:cb.m2m:flexContainer",      
}

# Result content indocators
RCIs = {
    
    'return-nothing':0,
    'rn':0,
    'return-all':1,
    'ra':1,
    'return-modified':9,
    'rm':9,
    
    'attributes':1,
    'a':1,
    'attributes-and-children':4,
    'ac':4,
    'children':8,
    'c':8
}

# Notification Content Types for subscriptions
NCTs = {
    "all":          1,
    "a":            1,
    "changed":      2,
    "c":            2,
    "all-and-meta": 3,
    "am":           3,
    "meta":         4,
    "m":            4

}

# Event Notification Criterias for subscriptions
ENCs = {
    "update":                1, #Resource updated
    "u":                     1,
    "delete":                2, #Resource deleted
    "d":                     2,
    "create-child":          3, # Create direct child resource
    "cc":                    3,
    "delete-child":          4, # Delete of direct child resource
    "dc":                    4,
    "retrieve-of-container": 5, # Retrieve of container resource
    "rc":                    5,               
    "command-completed":     6, # Command completed
    "coc":                   6,
    "interworking":          7, # Interworking in process
    "i":                     7
}

# Content encodings for messages posted into a container
CEs = {
  "none":   0,
  "base64": 1,
  "xml":    2,
  "json":   3,
  "opaque": 4, 
  "jpeg":   5,
  "gif":    6,
  "pdf":    1001
}
class CE(Enum):
  none   = 0
  base64 = 1
  xml    = 2
  json   = 3
  opaque = 4 
  jpeg   = 5
  gif    = 6
  pdf    = 1001
  
#pendingNotifications
PNs = {
    "sendLatest":     1,
    "sendAllPending": 2
}

# Operations types of device provisioning requests
DEVPROV_OPs = {
    "create": 1,
    "remove": 2
}



import json
import logging
from tkinter import *

log = logging.getLogger(__name__)

_LABEL_WIDTH = 12
_ENTRY_WIDTH = 60
_TEXT_WIDTH = 60
_FRAME_RELIEF = "flat"
_FRAME_BRDWIDTH = 1

def _createFrame(parent, relief=_FRAME_RELIEF, borderwidth=_FRAME_BRDWIDTH, padx=3, pady=3):
    return Frame(parent, relief=relief, padx=padx, pady=pady, borderwidth=borderwidth)

class InputEntry(Frame):
    def __init__(self, parent, 
                 labeltext, 
                 type="string", 
                 state=NORMAL, 
                 keyCallback=None):
        super().__init__(parent)

        self._type = type
        self._labeltext = labeltext

        if type == "string":
            self._var = StringVar()
        elif type == "int":
            self._var = IntVar()
        
        self._label = Label(self, text=labeltext, anchor="w", width=_LABEL_WIDTH)
        self._label.pack(side=LEFT)

        self._entry = Entry(self, textvariable=self._var, width=_ENTRY_WIDTH, state=state)
        self._entry.pack(side=LEFT, expand=True, fill=X)

        if keyCallback:
            self._entry.bind('<Key>', keyCallback)

        self.pack(side=TOP, expand=False, fill=X)

    @property
    def profileName(self):
        return self._profileName
    
    @property
    def type(self):
        return self._type
    
    @property
    def label(self):
        return self._labeltext
            
    def set(self, value):
        self._var.set(value)

    def get(self, inputRequired=False):
        return self._verify(inputRequired=inputRequired)
    
    def _verify(self, inputRequired=False):
        if self._type == "int":
            try:
                v = self._var.get()
                return v
            except Exception as ex:
                raise Exception("Invalid number for %s." % (self._labeltext))
        else:
            v = str(self._var.get()).strip()
            if inputRequired and not v or len(v) == 0:
                raise Exception("Input required for %s." % (self._labeltext))
            return v

class ProfileEditor(Frame):

    def __init__(self, parent, 
                 profileManager=None):
        super().__init__(parent)

        self._profileManager = profileManager
        self._cfg = None
        self._dirty = False
        self._new = True
        self._errorMsg = None
        self._profile = None

        inputFrame = _createFrame(self)
        
        self._statusVar = StringVar()
        self._statusLabel = self._buildStatusLabel(inputFrame, self._statusVar)
        self._updateStatusLabel()

        # Generate the input elements ...
        self._fileInput = InputEntry(inputFrame, "File", keyCallback=self.keyInputCallback, state=DISABLED)
        self._inputs = {}
        for c in self._profileManager.getProfileCfg()["elements"]:
            if c["type"] in ("string", "int"):
                i = InputEntry(inputFrame, c["label"], type=c["type"], state=c["state"], keyCallback=self.keyInputCallback)
            elif c["type"] == "text":
                i = self._buildInputText(inputFrame, "Application URL")                
            else:
                log.error(f'Unknown type {c["type"]} of profile element.')

            self._inputs[c["name"]] = i

        inputFrame.pack(side=TOP, expand=True, fill=BOTH)

        self._saveButton = None
        frame = self._buildCommandsFrame(self)
        frame.pack(side=BOTTOM, expand=False, fill=X)
    
    def _buildStatusLabel(self, parent, var):
        frame = _createFrame(parent, relief=GROOVE, borderwidth=2, padx=3, pady=1)
        label = Label(frame, textvariable=var, anchor="w")
        label.pack(side=TOP, expand=False, fill=X)
        frame.pack(side=TOP, expand=False, fill=X)
        return label

    def _buildInputText(self, parent, inputLabel):
        frame = _createFrame(parent)

        label = Label(frame, text=inputLabel, anchor="nw", width=_LABEL_WIDTH)
        label.pack(side=LEFT, expand=False, fill=Y)
        
        f = Frame(frame)
        f.pack(side=LEFT, expand=True, fill=BOTH)

        text = Text(f, height=10, width=_TEXT_WIDTH)
        text.pack(side=LEFT, expand=True, fill=BOTH)
        text.bind('<Key>', self.keyInputCallback)

        scrollV = Scrollbar(f)
        scrollV.pack(side = RIGHT, fill = Y)
        scrollV.config(command=text.yview)
        text.config(yscrollcommand=scrollV.set)

        frame.pack(side=TOP, expand=True, fill=BOTH)

        return text

    def _buildCommandsFrame(self, parent):
        frame = _createFrame(parent)
        self._saveButton = Button(frame, text="SAVE", command=self._saveProfile)
        self._saveButton.pack(side=RIGHT, padx = 10, pady = 10)
        return frame
            
    def _setErrorMsg(self, msg):
        self._errorMsg = msg

    def _updateStatusLabel(self):
        fg = "darkgrey"
        s = None
        color = None

        if self._errorMsg:
            s = "error: %s" % (self._errorMsg)
            fg = "red"
        else:
            if self._new:
                s = "new"
            else:
                s = "loaded"

            if self._dirty:
                s = "%s | changed" % (s)
            else:
                s = "%s | unchanged" % (s)

        self._statusVar.set(s)
        self._statusLabel.configure(fg=fg)

        # Clean error message
        self._setErrorMsg(None)

    def _setDirty(self, dirtyFlag):
        self._dirty = dirtyFlag
        self._updateStatusLabel()

    def keyInputCallback(self, event):
        self._setDirty(True)

    def _saveProfile(self):
        if not self._profile:
            return
        
        try: 
            for (k, i) in self._inputs.items():
                if type(i) == InputEntry:
                    self._profile.setElement(k, i.get(inputRequired=True), type=i.type)
                elif type(i) == Text:
                    d = json.loads(i.get(1.0, END).strip())
                    self._profile.setElement(k, d)
        except Exception as ex:    
            self._setErrorMsg(str(ex))
            self._updateStatusLabel()
            return

        try:
            self._profileManager.saveProfile(self._profile)
            self._new = False
            self._setDirty(False)

        except Exception as ex:
            self._setErrorMsg(str(ex))
            self._updateStatusLabel()
            return            

    def setProfile(self, profile):
        self._profile = profile
        self._fileInput.set("")

        # Cleanup
        for (k, i) in self._inputs.items():
            if type(i) == InputEntry:
                if i.type == "int":
                    i.set(0)
                elif i.type == "float":
                    i.set(0.0)
                else:
                    i.set("")
            elif type(i) == Text:
                i.delete(1.0, END)                
 
        self._new = True
        self._setDirty(False)

        if not profile:
            self._updateStatusLabel()
            return

        # Set the values from input profiile
        self._fileInput.set(profile.getFile())
         
        for (k, i) in self._inputs.items():
            if type(i) == InputEntry:
                if i.type == "int":
                    i.set(int(profile.getElement(k, 0)))
                elif i.type == "float":
                    i.set(float(profile.getElement(k, 0.0)))
                else:
                    i.set(profile.getElement(k, ""))
            elif type(i) == Text:
                d = profile.getElement(k, {})
                if d:
                    s = json.dumps(d, indent=4)
                else:
                    s = ""
                i.insert(1.0, s)

        self._new = False if profile.getFile() else True

        self._updateStatusLabel()





import logging
import os
import subprocess
import sys
import tempfile
import time
import webbrowser

import requests

import dtiot_d2c.cli.CFG as CFG

log = logging.getLogger(__name__)

def build_api_server_url()->str:
    return f"{CFG.PMAPI_PROTOCOL}://{CFG.PMAPI_HOST}:{CFG.PMAPI_PORT}/{CFG.PMAPI_PATH}"
    
def test_api_server_health()->bool:

    url = f"{build_api_server_url()}/healthz"
    log.debug(f"GET: {url}")

    headers = {
        "Content-Type"  : "application/json",
        "Accept"        : "application/json",
    }
    log.debug(f"HEADERS: {headers}")

    try:
        response = requests.get(url=url, headers=headers)   

        if response.ok:
            return True
        else:
            return False
    except Exception as ex:
        #log.error(f"Error while testing pm api {url}: {ex}")
        return False

def start_api_server(startup_timeout_secs:int=3)->bool:
    python_exe = sys.executable
    server_py = f"{os.path.dirname(__file__)}/prfmanapi/main.py"

    # Setze die Umgebungsvariable LOGLEVEL auf "debug"
    env = os.environ.copy()
    
    env["CFG"] = "DEFAULT"
    env["PRINTENV"] = "true"
    current_log_level = log.getEffectiveLevel()
    env["LOGLEVEL"] = logging.getLevelName(current_log_level)    

    # Create temp file for stdout and stderr
    temp_file, temp_file_name = tempfile.mkstemp()

    log.info(f"Starting profile manager API and deligating stdout and stderr to {temp_file_name}.")
    process = subprocess.Popen(
        [python_exe, server_py],
        env=env,
        start_new_session=True,
        stderr=temp_file,
        stdout=temp_file
    )
    
    # Wait until the server has been started ...
    before_time=time.time()
    while True:
        if test_api_server_health():
            break
        elif (time.time() - before_time) > startup_timeout_secs:
            log.error(f"Profile manager API doesn't response health.")
            return False
        else:
            time.sleep(0.5)

    return True
    
def open_url_in_browser(url):
    browsers = ['google-chrome', 'firefox', 'msedge', 'chrome', 'mozilla', 'edge']  
    for browser_name in browsers:
        try:
            browser = webbrowser.get(browser_name)
            browser.open(url)
            #print(f"URL geöffnet in {browser_name}.")
            return
        except webbrowser.Error as e:
           print(f"Konnte nicht öffnen mit {browser_name}: {e}")

    # Fallback auf Standardbrowser
    webbrowser.open(url)
    
def open_in_browser(webapp_url:str=None):
    
    ###
    # Test if the profile management api server is running.
    # If not, start it.    
    api_running=False
    wait_for_api_timeout_secs = 3
    start_time = time.time()
    while True:
        if test_api_server_health():
            api_running=True
            break
        elif (time.time() - start_time) > wait_for_api_timeout_secs:
            break
        else:
            time.sleep(1)

    # The api is not running, start it ...  
    if not api_running:
        if not start_api_server():
            raise Exception(f"Could not start profile manager API server locally.")

    log.info(f"Profile manager API server is running.")

    ###
    # Start profile manager web app
    
    ###
    # Open profile manager in the api
    log.info(f"Opening profile manager in browser ..")
    url = f"http://127.0.0.1:{CFG.PMWEBAPP_PORT}/{CFG.PMWEBAPP_PATH}"
    open_url_in_browser(url)
    #webbrowser.open_new(url)

    
    


import logging
from typing import List

from fastapi import HTTPException, Request

import dtiot_d2c.cli.prfmanapi.config as CFG

from . import api_utils

log = logging.getLogger(__name__)


def init():
    pass

def handleRequest(req:Request, id:str=None,
                 ) -> None | dict | List[str]:
    
    api_utils.logRequest(req)

    try:

        return ReportJob.get(id, when=when, month_of_year=month_of_year, 
                           from_date=from_date, to_date=to_date)
        
    except Exception as ex:
        log.error(f"Could not query job objectg from db: {ex}.")
        raise HTTPException(status_code=500, detail=f"Internal error.")







import json
import logging
from datetime import datetime
from typing import List

from pydantic import BaseModel, Field

import dtiot_d2c.d2c.consts as d2c_consts
import dtiot_d2c.dmo as dmo
from dtiot_d2c.d2c import utils as utils
from dtiot_d2c.d2c.application import Application
from dtiot_d2c.d2c.device import Device
from dtiot_d2c.d2c.onem2m_base import OneM2MBase
from dtiot_d2c.dmo import ApiClient

from .consts import *

log = logging.getLogger(__name__)

'''
{
    "dtiot:devGr": {
        "cnd": "com.telekom.iot.orchestrator.deviceGroup",
        "rn": "myDeviceGroup",
        "nds": [ "6724ca4407f20523ba2c6f1f","<applicatdevice_ri_2>", "...", "..." ],
        "aes": [ "66fe6b02baa471e374bd0041" ]
    }
} 
'''
class DeviceGroup(OneM2MBase):
    
    @property
    def devices(self)->List[str]:
        return self.onem2m.get("nds", []) 

    @devices.setter
    def devices(self, value:List[str]):
        current_list = self.devices
        self.onem2m["nds"] = utils.integrate_lists(current_list, value)

    @property
    def devicesCount(self)->int:
        return len(self.devices) if self.devices else 0
    
    @property
    def applications(self)->List[str]:
        return self.onem2m.get("aes", [])

    @applications.setter
    def applications(self, value:List[str]):
        current_list = self.applications
        self.onem2m["aes"] = utils.integrate_lists(current_list, value)

    @property
    def applicationsCount(self)->int:
        return len(self.applications) if self.applications else 0        

    @property
    def description(self):
        return self._get_label(f"{DESCRIPTION_LABEL_NAME}")

    @description.setter
    def description(self, value): 
        labels = {f"+{DESCRIPTION_LABEL_NAME}":value}
        self.integrate_labels(labels)
        
    def model_dump(self,  *args, **kwargs):
        md = super().model_dump(*args, **kwargs)
        for key in ["description",
                    "devices", 
                    "devicesCount",
                    "applications", 
                    "applicationsCount",
                   ]:
            md[key] = getattr(self, key)
        del md["labels"]
        return md
        
    def set_from_kwargs(self, **kwargs):
        super().set_from_kwargs(**kwargs)
        
        #### Copy simple string argumentsw
        for (paramName, attrName) in (("description", "description"),
                                     ):
            if not (value := kwargs.get(paramName, None)):
                continue
            setattr(self, attrName, value)
                    
        # Copy list args into the object
        for (paramName, attrName) in [["devices", "devices"],
                                      ["applications", "applications"],
                                     ]:
            if not (value := kwargs.get(paramName, None)):
                continue

            if type(value) == list:
                setattr(self, attrName, value)
            elif type(value) == str:
                setattr(self, attrName, utils.splitString(value, ","))
                #setattr(self, key, json.loads(value))
            else:
                log.error(f"Unsupported type {type(value)} for argument {paramName}={value}")

                  
    def to_onem2m(self, **kwargs)->dict:
        d = {}
        for (attr_name, onem2m_path) in [["devices",      "nds"],
                                         ["applications", "aes"],
                                        ]:
            d[onem2m_path] = getattr(self, attr_name)

        d["lbl"] = dmo.convert_dict_to_onem2m_labels(self.labels)
        
        # Add the d2c version
        d2c_version_lbl = f"{d2c_consts.D2C_SYSTEM_VERSION_LABEL}:{d2c_consts.D2C_SYSTEM_VERSION}"
        if d2c_version_lbl not in d["lbl"]:
            d["lbl"].append(d2c_version_lbl)        

        # add the additional keyword arguments
        for (key, value) in kwargs.items():
            d[key] = value
            
        return {"dtiot:devGr":d}

    def add_devices(self, add_names:List[str])->List[str]:
        if not add_names:
            return 
        
        current_names = self.devices
        for name in add_names:
            if name not in current_names:
                current_names.append(name)

    def remove_devices(self, rm_names:List[str])->List[str]:
        if not rm_names:
            return
        current_names = self.devices
        for name in rm_names:
            try:
                current_names.remove(name)
            except Exception:
                log.warning(f"Device name {name} not in current list of devices.")
        
    def add_applications(self, add_names:List[str])->List[str]:
        if not add_names:
            return 
        
        current_names = self.applications
        for name in add_names:
            if name not in current_names:
                current_names.append(name)

    def remove_applications(self, rm_names:List[str])->List[str]:
        if not rm_names:
            return 
        
        current_names = self.applications
        for name in rm_names:
            try:
                current_names.remove(name)
            except Exception:
                log.warning(f"Application name {name} not in current list of applications.")
        
    ######################################################################
    # Static public functions
    ######################################################################
    def get_create_onem2m_dict(name:str, **kwargs)->dict:
        
        labels = kwargs.get("labels", {})
        labels[d2c_consts.D2C_SYSTEM_VERSION_LABEL] = d2c_consts.D2C_SYSTEM_VERSION
        labels[DESCRIPTION_LABEL_NAME] = kwargs.get("description", None)
        
        return {
            "dtiot:devGr": {
                "cnd": DEVICE_GROUP_CND,
                "rn": name,
                "lbl": dmo.convert_dict_to_onem2m_labels(labels),
                "nds": kwargs.get("devices", []),
                "aes": kwargs.get("applications", [])
            }
        }        
            
    def get(api_client:ApiClient,  
            origin:str=None,    
            id:str=None,   
            name:str=None,
            resultContentIndocator:str=None, # Result content indicator. Supported types are RCIs (see above).
            limit:int=1000000000,            # Max number of objects to return.
            offset:int=0,
            select:str=None,                 # Comma separated list of element names which shall be selected.
            format:str="CSV"                 # Output format. Possible values are CSV and JSON
           ):
        return get(api_client, 
                   origin=origin, 
                   id=id, name=name, 
                   limit=limit, offset=offset,
                   resultContentIndocator=resultContentIndocator,
                   select=select, format=format)
        
    def create(api_client:ApiClient,
               origin:str=None,     
               name:str=None,
               **kwargs):    
        return create(api_client, origin, name, **kwargs)
    
    def delete(api_client:ApiClient, 
               origin:str=None, 
               id:str=None,
               name:str=None, 
              ):
        return delete(api_client, origin=origin, id=id, name=name)
    
    def update(api_client:ApiClient, origin:str=None, 
               device_group=None,  # DeviceGroup
               id:str=None,
               name:str=None,
               **kwargs
              ):  
        return update(api_client, origin, 
                           device_group=device_group, 
                           id=id,
                           name=name,
                           **kwargs)  
        
    def update_devices(api_client:ApiClient, 
                       origin:str=None, 
                       device_group=None,  # DeviceGroup
                       name:str=None,
                       add_names:List[str]=None,
                       remove_names:List[str]=None,
                     ):  
        return update_devices(api_client, origin, device_group=device_group, name=name,
                              add_names=add_names, remove_names=remove_names,
                             )
        
    def update_applications(api_client:ApiClient, 
                            origin:str=None, 
                            device_group=None,  # DeviceGroup
                            name:str=None,
                            add_names:List[str]=None,
                            remove_names:List[str]=None,
                          ):  
        return update_applications(api_client, origin=origin, device_group=device_group, name=name,
                                   add_names=add_names, remove_names=remove_names,
                                  )        

################################################################
################################################################
################################################################
#MARK: IMPLEMENTATIONS
def _get(api_client:ApiClient,  # Client connection definition.
         origin:str=None,       # Value for X-M2M-Origin header 
         id:str=None,
         name:str=None,
         resultContentIndocator:str=None, # Result content indicator. Supported types are RCIs (see above).
         limit:int=1000000000,            # Max number of objects to return.
         offset:int=0,
         select:str=None,                 # Comma separated list of element names which shall be selected.
         format:str="CSV"                 # Output format. Possible values are CSV and JSON
        ):        
    if not name and not id:
        response = dmo.get_resources(api_client, 
                                     origin=origin,
                                     limit=limit,
                                     offset=offset,
                                     resourceType="fcn", 
                                     resourceName="device-group",
                                     resultContentIndocator=resultContentIndocator)
    else:
        response = dmo.get_resources(api_client, 
                                     origin=origin,
                                     resourceType="fcn", 
                                     resourceId=id,
                                     resourceName=f"device-group/{name}", 
                                     resultContentIndocator=resultContentIndocator)      
    if not response:
        return None
    
    if not name and not id:
        object = [DeviceGroup(onem2m=onem2m) for onem2m in response.get("dtiot:devGr", [])]
    else:
        object = DeviceGroup(onem2m=response)  
                
    # Return the plain objects
    if not select:
        return object
    else:
        return dmo.select_from_response_body(object, select=select, format=format)

def get(api_client:ApiClient,  # Client connection definition.
        origin:str=None,       # Value for X-M2M-Origin header 
        id:str=None,
        name:str=None,
        resultContentIndocator:str=None, # Result content indicator. Supported types are RCIs (see above).
        limit:int=1000000000,            # Max number of objects to return.
        offset:int=0,
        select:str=None,                 # Comma separated list of element names which shall be selected.
        format:str="CSV"                 # Output format. Possible values are CSV and JSON
       ):        

    if name or id:
        log.info(f"Getting device group {name or id} from DMO ...")
    else:
        log.info(f"Getting device groups from DMO ...")

    return _get(api_client, origin=origin, id=id, name=name, 
                limit=limit, offset=offset, 
                select=select, format=format, 
                resultContentIndocator=resultContentIndocator)
        
def create(api_client:ApiClient, 
           origin:str=None,      
           name:str=None,
           **kwargs):

    log.info(f"Creating device group {name} in DMO ...")
    onem2m = DeviceGroup.get_create_onem2m_dict(name, **kwargs)
    dmo.add_resource(api_client, origin=origin, r_type="fcn", r_name="device-group", 
                     onem2m_body=onem2m)

    return get(api_client, origin=origin, name=name)

def delete(api_client:ApiClient, 
           origin:str=None, 
           id:str=None,
           name:str=None, 
          ):
    if not name and not id:
        raise Exception(f"delete_device_group() requires parameter name or id as input.")
    
    log.info(f"Deleting device group {name or id} from DMO ...")
    
    if id:
        resource_name = None
    else:
        resource_name = f"device-group/{name}"
    return dmo.delete_resource(api_client, origin=origin, name=resource_name, id=id)

def _update(api_client:ApiClient,  # Client connection definition.
            origin:str,            # Value for X-M2M-Origin header 
            device_group:DeviceGroup,
           )->DeviceGroup:
    if not device_group:
        raise Exception(f"update_device_group() requires parameter device_group as input.")

    # Create the oneM2M dvi dictionary from the device
    onem2m:dict = device_group.to_onem2m()

    log.info(f"Updating device group {device_group.name} in DMO ...")        
    dmo.update_resource(api_client, origin, name=f"device-group/{device_group.name}", onem2m_body=onem2m)

    # Reload the device group and return it
    return get(api_client, origin=origin, id=None, name=device_group.name)  

def update(api_client:ApiClient,  
           origin:str=None,       
           device_group:DeviceGroup=None,
           id:str=None,
           name:str=None,
           **kwargs
          ):
    if device_group:
        return _update(api_client, origin=origin, device_group=device_group)
    elif not name and not id:
        raise Exception(f"update_device_group() requires parameter device_group, name or id as input.")

    device_group = get(api_client, origin=origin, id=id, name=name)  

    # Copy the keyword arguments into the device object
    device_group.set_from_kwargs(**kwargs)
            
    return _update(api_client,origin=origin, device_group=device_group)

def update_devices(api_client:ApiClient,  
                   origin:str=None,       
                   device_group:DeviceGroup=None,
                   name:str=None,
                   add_names:List[str]=None,
                   remove_names:List[str]=None,
                  ):
    if not device_group and not name:
        raise Exception(f"update_devices() requires parameter device_group or name as input.")

    if not device_group:
        device_group = get(api_client, origin=origin, name=name)
        
    device_group.add_devices(add_names)
    device_group.remove_devices(remove_names)

    return _update(api_client, origin, device_group)


def update_applications(api_client:ApiClient,  
                        origin:str=None,       
                        device_group:DeviceGroup=None,
                        name:str=None,
                        add_names:List[str]=None,
                        remove_names:List[str]=None,
                       ):
    if not device_group and not name:
        raise Exception(f"update_applications() requires parameter device_group or name as input.")

    if not device_group:
        device_group = get(api_client, origin=origin, name=name)
        
    device_group.add_applications(add_names)
    device_group.remove_applications(remove_names)

    return _update(api_client, origin, device_group)



import json
import logging
import sys
from argparse import ArgumentParser

from dtiot_d2c.cli.cli_command import ApiConnCmdLineInterface, CmdLineInterface

log = logging.getLogger(__name__)
            
import dtiot_d2c.d2c as d2c


#####################################################################################                
class VersionCLI(CmdLineInterface):

    def __init__(self):
        super().__init__()

    def get1LineHelp(self):
        return "Prints the version infos."
    
    def addCmdLineArgs(self, argsParser):
        pass

    def main(self, cmdargs=None):
        print(json.dumps(d2c.get_version_info(), indent=4))

        

            
            
            



import dtiot_d2c.d2c.utils as utils
import dtiot_d2c.dmo as dmo

D2C_SYSTEM_VERSION_LABEL = "d2c"
D2C_SYSTEM_VERSION = "1.0"

# Operation indicator characters to update labels
UPDT_LABS_OPERATION_INDICATOR_ADD = "+"
UPDT_LABS_OPERATION_INDICATOR_DEL = "-"
UPDT_LABS_OPERATION_INDICATOR_ADD = "+"
UPDT_LABS_OPERATION_INDICATOR_DEL = "-"

CMDARGS:dict = {
    "-o": ["--origin", 
           {"metavar":"<string>", 
            "dest":"origin", 
            "required": False,
            "default":None,
            "help":"X-M2M-ORIGIN heder."}
    ],
    "-n" : ["--name", 
        {
            "dest":"name", 
            "metavar":"<string>", 
            "help":f"Unique object name."
        }
    ],      
    "-i" : ["--id", 
        {
            "dest":"id", 
            "metavar":"<string>", 
            "help":f"Id of the object."
        }
    ],          
    "-lbl" : ["--labels", 
        {
            "metavar":"{key:value, key:value, ...}", 
            "dest":"labels", 
            "required":False,
            "default":None,
            "help":"Labels defined by a JSON dictionary of key/value pairs."
        }
    ],   
    # Device Group parameters 
    "-ds": ["--devices", 
            {
                "type": utils.argparse_type_list,
                "metavar":"<list-of-strings>", 
                "dest":"devices",
                "required": False,
                "default":[],
                "help":"Comma separated list of onem2m device names."
            }
    ],            
    "-as": ["--applications", 
            {
                "type": utils.argparse_type_list,
             "metavar":"<list-of-strings>", 
            "dest":"applications",
            "required": False,
            "default":[],
            "help":"Comma separated list of onem2m application names."
        }
    ],     
    "-addns": ["--add-names", 
            {"metavar":"<string>", 
            "dest":"add_names",
            "required": False,
            "default":[],
            "help":"Comma separated list of device or application names which shall be added."}
    ],         
    "-rmns": ["--remove-names", 
            {"metavar":"<string>", 
            "dest":"remove_names",
            "required": False,
            "default":[],
            "help":"Comma separated list of device or application names which shall be remvoved."}
    ],         

    # Application parameters
    "-us" : ["--urls", 
        {
            "type": utils.argparse_type_list,
            "dest":"urls", 
            "metavar":"<urls>",
            "help":f"Comma sepaerated list of application urls. "
        }
    ],         
    "-at" : ["--application-type", 
        {
            "metavar":"<string>", 
            "dest":"application_type", 
            "choices": ["webHook"], 
            "default":"webHook", 
            "help":f"Type of the application. Possible values are webHook. If not defined webHook is used."
        }
    ],         
    "-cp" : ["--connection-properties", 
        {
            "type":utils.argparse_type_json,
            "dest":"connection_properties", 
            "metavar":"'<{\"key\":\"value\", \"key\":\"value\"}>'", 
            "help":f"Properties to configure the application connection. For WebHooks it would be header fields."
        }
    ],      

    # Device parameters
    "-rt": ["--type", 
            {"metavar":"<string>",
            "dest": "ty",
            "choices": dmo.TYs.keys(),
            "help":"Type of resource."}
    ],      
    "-ma" : ["--manufacturer", 
        {
            "metavar":"<string>", 
            "dest":"manufacturer", 
            "required":False,
            "default":None,
            "help":f"Manufacturer of the device. If not defined Generic is used."
        }
    ],    
    "-mo" : ["--model", 
        {
            "metavar":"<string>", 
            "dest":"model", 
            "required":False,
            "default":None,
            "help":f"Model of the device. If not defined Generic is used."
        }
    ],    
    "-smo" : ["--sub-model", 
        {
            "metavar":"<string>", 
            "dest":"sub_model", 
            "required":False,
            "help":f"Sub model of the device."
        }
    ],        
    "-ic" : ["--iccid", 
        {
            "metavar":"<string>", 
            "dest":"iccid", 
            "required":False,
            "default":"282828282828", 
            "help":f"ICCID of the SIM card in the device. If not defined 282828282828 is used."
        }
    ],    
    "-pr" : ["--protocol", 
        {
            "metavar":"<string>", 
            "dest":"protocol", 
            "choices": ["LWM2M", "LoRaWAN", "UDP", "MQTT", "DTLS CoAP", "CoAP", "HTTP"], 
            "default":"LWM2M", 
            "help":f"Protocol of the device. Possible values are HTTP, MQTT, LWM2M. If not defined LWM2M is used."
        }
    ],        
    "-lo" : ["--location", 
        {
            "dest":"location", 
            "metavar":"<string>", 
            "help":f"Device location."
        }
    ],        
    "-la" : ["--label", 
        {
            "dest":"label", 
            "metavar":"<string>", 
            "help":f"Device label."
        }
    ],        
    "-fv" : ["--firmware-version", 
        {
            "dest":"firmware_version", 
            "metavar":"<string>", 
            "help":f"Device firmware version."
        }
    ],    
    "-sv" : ["--software-version", 
        {
            "dest":"software_version", 
            "metavar":"<string>", 
            "help":f"Device firmware version."
        }
    ],        
    "-ov" : ["--os-version", 
        {
            "dest":"os_version", 
            "metavar":"<string>", 
            "help":f"Device operating system version."
        }
    ],    
    "-hv" : ["--hardware-version", 
        {
            "dest":"hardware_version", 
            "metavar":"<string>", 
            "help":f"Device hardware version."
        }
    ],        
    "-cy" : ["--country", 
        {
            "dest":"country", 
            "metavar":"<string>", 
            "default":None,
            "help":f"Device country code. If not defined DE is used."
        }
    ],        
    "-dty" : ["--device-type", 
        {
            "dest":"device_type", 
            "metavar":"<string>", 
            "help":f"Type of the device."
        }
    ],      
    "-dna" : ["--device-name", 
        {
            "dest":"device_name", 
            "metavar":"<string>", 
            "help":f"Type of the device."
        }
    ],      
    "-de" : ["--description", 
        {
            "dest":"description", 
            "metavar":"<string>", 
            "help":f"Description of the device."
        }
    ],   
    "-up" : ["--uplink-properties", 
        {
            "dest":"uplink_properties", 
            "metavar":"'<{\"key\":\"value\", \"key\":\"value\"}>'", 
            "help":f"Additional uplink properties."
        }
    ],   
    "-dp" : ["--device-properties", 
        {
            "dest":"device_properties", 
            "metavar":"'<{\"key\":\"value\", \"key\":\"value\"}>'", 
            "help":f"Additional device properties."
        }
    ],   
    "-cid" : ["--credentials-id", 
        {
            "dest":"credentials_id", 
            "metavar":"'<string>'", 
            "help":f"Id of the device credentials such as PSK identity. If this is not defined the device serial number is used instead."
        }
    ],   
    "-csec" : ["--credentials-secret", 
        {
            "dest":"credentials_secret", 
            "metavar":"'<string>'", 
            "help":f"Secret of the device credentials such as the PSK key for UDP encryption. This key shall be base64 encoded."
        }
    ],
    "-mn" : ["--message-name", 
        {
            "dest":"message_name", 
            "metavar":"<string>", 
            "help":f"Identifying name of the message."
        }
    ],
    "-mi" : ["--message-id", 
        {
            "dest":"message_id", 
            "metavar":"<string>", 
            "help":f"Identifying id of the message."
        }
    ],     
    "-ms" : ["--message-store", 
        {
            "metavar":"<message-store>", 
            "dest":"msg_store", 
            "choices": ["uplink-inbound", "uli", "uplink-outbound", "ulo",
                        "downlink-inbound", "dli", "downlink-outbound", "dlo"], 
            "help": "Type of message store which shall be queried. Possible values are: uplink-inbound, uli, uplink-outbound, ulo, downlink-inbound, dli, downlink-outbound, dlo."
        }
    ],            
}     


import logging
from datetime import datetime
from typing import List, Literal

import dtiot_d2c.dmo.utils as utils
from dtiot_d2c.dmo.consts import *

log = logging.getLogger(__name__)

def convert_dict_to_onem2m_labels(d:dict)->List[str]:
    """Converts a dcitionary to onem2m labels

    Args:
        d (dict): Dictionary which shall be converted.

    Returns:
        List[str]: Array of strings of the form ["<key>:<value>", "<key>:<value>", ...]
    """
    if not d:
        return []
    
    labels = []
    for (k, v) in d.items():
        if v == None:
            v = ""
        
        if v and type(v) == str:
            v = v.replace(":", " ")
        labels.append(f"{k}:{v}")            
    return labels

def convert_onem2m_labels_to_dict(labels:List[str])->dict:
    if not labels:
        return {}
    
    d={}
    for label in labels:
        idx = label.find(":")
        if idx == -1:
            continue
        d[label[0:idx]] = utils.convert_str_to_python_type(label[idx+1:])
    return d

def convert_onem2m_timestamp_to_datetime(onem2m_ts:str= "20250101T000000,000000")->datetime:
    # "ct": "20250321T095914,759000",
    if onem2m_ts and (idx:=onem2m_ts.find(",")) > -1:
        onem2m_ts = onem2m_ts[0:idx]        

    t_date:datetime = None
    
    try:
        t_date = datetime.strptime(onem2m_ts, ONEM2M_DATE_FORMAT)
    except Exception as ex:
        log.error(f"Cannot convert creation date '{onem2m_ts}' into date objekt: {ex}")
        t_date = datetime.now()

    return t_date

def select_from_response_body(body, 
                              select:List[str] | str = None, 
                              format:Literal["JSON", "CSV"] = "CSV"
                             ):
    """_summary_

    Args:
        body (_type_): _description_
        select (List[str] | str, optional): _description_. Defaults to None.

    Raises:
        Exception: _description_
        Exception: _description_

    Returns:
        _type_: _description_
    """

    if not select:
        return body

    # dicts    
    dicts = []
    is_object = False
    
    if type(body) == dict:
        dicts.append(body)
    elif type(body) == list:
        for obj in body:
            if type(obj) == dict:
                dicts.append(obj)
            else:
                dicts.append(obj.dict())
    else:
        dicts.append(body.dict())
        is_object = True
    
    # select
    if type(select) == str:
        selectKeys = utils.splitString(select, ",")
    elif type(select) == list:
        selectKeys = select
    else:
        raise Exception(f"Unsupported type {type(select)} for select parameter {select}.")

    # format   
    if not format:
        format = "CSV"
    if format not in ["CSV", "JSON"]:
        raise Exception(f"Unsupported format {format}.")

    # Filter elements from body data
    filtered_elems = []
    for d in dicts:
        filtered_elem = {}        
        for key in selectKeys:
            filtered_elem[key] = utils.getValueFromDict(d, key, "", sepChar=".")
        filtered_elems.append(filtered_elem)

    if format.upper() == "CSV":
        lines = []
        for elem in filtered_elems:
            line = []
            for (key, value) in elem.items():
                line.append(str(value))
            lines.append(line)
        return lines if not is_object else lines[0]

    elif format.upper() == "JSON":
        return filtered_elems if not is_object else filtered_elems[0]

    else:
        raise Exception(f"Unsupported format {format}. JSON or CSV required.") 
            



# Device group container definition     
DEVICE_GROUP_CND = "com.telekom.iot.orchestrator.deviceGroup"

# Type label name
APPL_TYPE_LABEL_NAME = "type"

# Webhook application type 
APPL_TYPE_WEBHOOK = "webHook"

# Default application type
APPL_TYPE_DEFAULT = APPL_TYPE_WEBHOOK

# Label prefixs for the type specific connection properties
CONN_PROPS_LABEL_PREFIXS = {
    APPL_TYPE_WEBHOOK: "WebHookHttpHeader_"
}

# Description label name
DESCRIPTION_LABEL_NAME = "description"



import os

###
# Feature flags of the various CLI command interfaces. Those flags are set by 
# the command implementation to tell the controlling modules what the implemenation
# requires.
###
REQUIRES_CONFIG = 1
REQUIRES_PROFILE = 2
REQUIRES_DMOAPICONNECT = 4

# Names of environment variables, directories and files
###

# Environment variable of the directory which contains the configuration files
ENVVARNAME_CFGDIR = "D2CCLI_CONFIGDIR"

# File which contains the API access configuration
ENVVARNAME_CFGFILE = "D2CCLI_CONFIGFILE"

# Default name of the directory which contains the configuration files
DEFAULT_CFGDIR_NAME = ".d2c"

# Default name of the configuration file
DEFAULT_CFGFILE_NAME = "config.json"

# Name of the profiles sub directory within the configuration directory
DEFAULT_PROFILES_DIRNAME = "profiles"
DEFAULT_PROFILES_DIRPATH = "$HOME/%s/%s" % (DEFAULT_CFGDIR_NAME, DEFAULT_PROFILES_DIRNAME)

# Profile file extension
PROFILE_FILE_EXT = "profile"

# Configuration element names
CEN_ACTIVE_PROFILE_FILE = "activeProfileFile"
CEN_PROFILE_DIR = "profilesDir"

# Init configuration 
DEFAULT_CONFIGURATION = {
    "activeProfileFile": "",
    "profilesDir": DEFAULT_PROFILES_DIRPATH
}

# Profile configuration
PROFILE_CONFIG = {
    "elements": [
        {"name":"profileName",  "label":"Profile Name",       "type":"string", "state":"normal", "default":"My Profile"},
        {"name":"apiUrl",       "label":"API URL",             "type":"string", "state":"normal", "default":"https://qa.spacegate.telekom.de/bond/t-iot-hub/device-management-orchestrator/v3/TENANT-REQUIRED"},
        {"name":"getTokenUrl",  "label":"Get Token URL",       "type":"string", "state":"normal", "default":"https://qa.spacegate.telekom.de/auth/realms/bond/protocol/openid-connect/token"},
        {"name":"clientId",     "label":"Client Id",          "type":"string", "state":"normal", "default":"t-iot-hub--##########--########-####-####-####-################"},
        {"name":"clientSecret", "label":"Client Secret",      "type":"string", "state":"normal", "default":"########-####-####-####-########"},
    ],
    "filenameElements":["profileName", ".profile"] 
}

D2CCLI_DEFAULT_ORIGIN="d2ccli"

###
# Factory function to create an APIConnection object from profile
from dtiot_d2c.cli.profile import Profile


def createApiConnection(profile:Profile)->any:
    from dtiot_d2c.dmo import ApiClient
    apiClient:ApiClient = ApiClient(api_url=profile.getElement("apiUrl"),
                                    origin=D2CCLI_DEFAULT_ORIGIN,
                                    get_token_url=profile.getElement("getTokenUrl"),
                                    client_id=profile.getElement("clientId"),
                                    client_secret=profile.getElement("clientSecret"))

    apiClient.auth()
    
    return apiClient

###
# Configuration of the profile manager web application and api
# Profile management web application
#PMWEBAPP_PORT=3000
# PMWEBAPP_PATH="/profile-manager"

# Profile management API 
#from .prfmanapi.configs import DEFAULT as PMAPI_CFG
#PMAPI_PROTOCOL="http"
#PMAPI_HOST="127.0.0.1"
#PMAPI_PORT=PMAPI_CFG.ServicePort
#PMAPI_PATH=PMAPI_CFG.ServiceName

###
# Registration of cli commands ...
CLICommands = [
    {
        "name": "CONFIGURATION",
        "help": "Confguration commands",
        "commands": [
            {
                "name": ["create-configuration", "cc"],
                "cmdFile": "%s/configCLI.py" % (os.path.dirname(__file__)),
                "cmdClass": "CreateConfigurationCLI"
            }
        ]
    },
    {
        "name": "PROFILE",
        "help": "Profile commands",
        "commands": [
            {
                "name": ["list-profiles", "lp"],
                "cmdFile": "%s/profileCLI.py" % (os.path.dirname(__file__)),
                "cmdClass": "ListProfilesCLI"
            },
            {
                "name": ["print-profile", "pp"],
                "cmdFile": "%s/profileCLI.py" % (os.path.dirname(__file__)),
                "cmdClass": "PrintProfileCLI"
            },
            {
                "name": ["add-profile", "ap"],
                "cmdFile": "%s/profileCLI.py" % (os.path.dirname(__file__)),
                "cmdClass": "NewProfileCLI"
            },
            {
                "name": ["modify-profile", "mp"],
                "cmdFile": "%s/profileCLI.py" % (os.path.dirname(__file__)),
                "cmdClass": "EditProfileCLI"
            },
            {
                "name": ["activate-profile", "acp"],
                "cmdFile": "%s/profileCLI.py" % (os.path.dirname(__file__)),
                "cmdClass": "ActivateProfileCLI"
            },
            {
                "name": ["print-active-profile", "pap"],
                "cmdFile": "%s/profileCLI.py" % (os.path.dirname(__file__)),
                "cmdClass": "PrintActivatedProfileCLI"
            },
            {
                "name": ["profile-manager", "pm"],
                "cmdFile": "%s/profileCLI.py" % (os.path.dirname(__file__)),
                "cmdClass": "StartProfileManagerGuiCLI"
            },
            # {
            #     "name": ["profile-manager-webapp", "pm2"],
            #     "cmdFile": "%s/profileCLI.py" % (os.path.dirname(__file__)),
            #     "cmdClass": "StartProfileManagerWebBrowserCLI"
            # }
            
        ]
    },
    {
        "name": "DMO",
        "help": "DMO functions",
        "commands": [
            {
                "name": ["get-resources", "gr"],
                "cmdFile": "%s/../dmo/dmoCLI.py" % (os.path.dirname(__file__)),
                "cmdClass": "GetResourcesCLI"
            },
            {
                "name": ["add-resource", "ar"],
                "cmdFile": "%s/../dmo/dmoCLI.py" % (os.path.dirname(__file__)),
                "cmdClass": "AddResourceCLI"
            },
            {
                "name": ["update-resource", "ar"],
                "cmdFile": "%s/../dmo/dmoCLI.py" % (os.path.dirname(__file__)),
                "cmdClass": "UpdateResourceCLI"
            },
            {
                "name": ["delete-resource", "dr"],
                "cmdFile": "%s/../dmo/dmoCLI.py" % (os.path.dirname(__file__)),
                "cmdClass": "DeleteResourceCLI"
            },
            {
                "name": ["get-device-provisioning-requests", "gdpreq"],
                "cmdFile": "%s/../dmo/dmoCLI.py" % (os.path.dirname(__file__)),
                "cmdClass": "GetDeviceProvisioningRequestsCLI"
            },                      
            {
                "name": ["get-device-provisioning-responses", "gdpres"],
                "cmdFile": "%s/../dmo/dmoCLI.py" % (os.path.dirname(__file__)),
                "cmdClass": "GetDeviceProvisioningResponsesCLI"
            },      
            # {
            # {
            #     "name": ["update-resource", "ur"],
            #     "cmdFile": "%s/../iothubcli/dmoCLI.py" % (os.path.dirname(__file__)),
            #     "cmdClass": "UpdateResourceCLI"
            # },
            # {
            #     "name": ["add-message-container", "amsgc"],
            #     "cmdFile": "%s/../iothubcli/dmoCLI.py" % (os.path.dirname(__file__)),
            #     "cmdClass": "AddMessageContainerCLI"
            # },                 
            # {
            #     "name": ["add-subscription", "asub"],
            #     "cmdFile": "%s/../iothubcli/dmoCLI.py" % (os.path.dirname(__file__)),
            #     "cmdClass": "AddSubscriptionCLI"
            # },
            # {
            #     "name": ["post-message", "pmsg"],
            #     "cmdFile": "%s/../iothubcli/dmoCLI.py" % (os.path.dirname(__file__)),
            #     "cmdClass": "PostMessageToContainerCLI"
            # },
        ]
    },
    # Devices
    {
        "name": "D2C Device Management",
        "help": "D2C functions to manage devices",
        "commands": [

            {
                "name": ["add-device", "ad"],
                "cmdFile": "%s/../d2c/device/cli.py" % (os.path.dirname(__file__)),
                "cmdClass": "AddDeviceCLI"
            },    
            {
                "name": ["add-lwm2m-device", "alwm2md"],
                "cmdFile": "%s/../d2c/device/cli.py" % (os.path.dirname(__file__)),
                "cmdClass": "AddLwm2mDeviceCLI"
            },                
            {
                "name": ["get-devices", "gd"],
                "cmdFile": "%s/../d2c/device/cli.py" % (os.path.dirname(__file__)),
                "cmdClass": "GetDevicesCLI"
            },      
            {
                "name": ["update-device", "ud"],
                "cmdFile": "%s/../d2c/device/cli.py" % (os.path.dirname(__file__)),
                "cmdClass": "UpdateDeviceCLI"
            },
            {
                "name": ["delete-device", "dd"],
                "cmdFile": "%s/../d2c/device/cli.py" % (os.path.dirname(__file__)),
                "cmdClass": "DeleteDeviceCLI"
            },
            {
                "name": ["inject-uplink-message", "ium"],
                "cmdFile": "%s/../d2c/device/cli.py" % (os.path.dirname(__file__)),
                "cmdClass": "InjectUplinkMessageCLI"
            },                      
            {
                "name": ["inject-downlink-message", "idm"],
                "cmdFile": "%s/../d2c/device/cli.py" % (os.path.dirname(__file__)),
                "cmdClass": "InjectDownlinkMessageCLI"
            },                      
            {
                "name": ["get-device-messages", "gdms"],
                "cmdFile": "%s/../d2c/device/cli.py" % (os.path.dirname(__file__)),
                "cmdClass": "GetDeviceMessagesCLI"
            },            
            {
                "name": ["delete-device-message", "ddm"],
                "cmdFile": "%s/../d2c/device/cli.py" % (os.path.dirname(__file__)),
                "cmdClass": "DeleteDeviceMessageCLI"
            },            
            
        ]
    },
    # Applications
    {
        "name": "D2C Application Management",
        "help": "D2C functions to manage applications",
        "commands": [    
            {
                "name": ["add-application", "aa"],
                "cmdFile": "%s/../d2c/application/cli.py" % (os.path.dirname(__file__)),
                "cmdClass": "AddApplicationCLI"
            },    
            {
                "name": ["get-applications", "ga"],
                "cmdFile": "%s/../d2c/application/cli.py" % (os.path.dirname(__file__)),
                "cmdClass": "GetApplicationsCLI"
            },                              
            {
                "name": ["update-application", "ua"],
                "cmdFile": "%s/../d2c/application/cli.py" % (os.path.dirname(__file__)),
                "cmdClass": "UpdateApplicationCLI"
            },                        
            {
                "name": ["delete-application", "da"],
                "cmdFile": "%s/../d2c/application/cli.py" % (os.path.dirname(__file__)),
                "cmdClass": "DeleteApplicationCLI"
            },   
        ]
    },
    ### Device Groups
    {
        "name": "D2C Device Group Management",
        "help": "D2C functions to manage device groups",
        "commands": [
            {
                "name": ["add-device-group", "adg"],
                "cmdFile": "%s/../d2c/device_group/cli.py" % (os.path.dirname(__file__)),
                "cmdClass": "AddDeviceGroupCLI"
            },                       
            {
                "name": ["get-device-groups", "gdg"],
                "cmdFile": "%s/../d2c/device_group/cli.py" % (os.path.dirname(__file__)),
                "cmdClass": "GetDeviceGroupsCLI"
            },                              
            {
                "name": ["update-device-group", "udg"],
                "cmdFile": "%s/../d2c/device_group/cli.py" % (os.path.dirname(__file__)),
                "cmdClass": "UpdateDeviceGroupCLI"
            },                        
            {
                "name": ["delete-device-group", "ddg"],
                "cmdFile": "%s/../d2c/device_group/cli.py" % (os.path.dirname(__file__)),
                "cmdClass": "DeleteDeviceGroupCLI"
            },     
            {
                "name": ["device-group-update-devices", "dguds"],
                "cmdFile": "%s/../d2c/device_group/cli.py" % (os.path.dirname(__file__)),
                "cmdClass": "ModifyDevicesCLI"
            },                 
            {
                "name": ["device-group-update-applications", "dguas"],
                "cmdFile": "%s/../d2c/device_group/cli.py" % (os.path.dirname(__file__)),
                "cmdClass": "ModifyApplicationsCLI"
            }, 
        ]
    },
    # Admin
    {
        "name": "D2C Admin",
        "help": "D2C admin functions",
        "commands": [
            {
                "name": ["add-d2ccli-application"],
                "cmdFile": "%s/../d2c/application/cli.py" % (os.path.dirname(__file__)),
                "cmdClass": "AddC2cCliApplicationCLI"
            },                
            {
                "name": ["version"],
                "cmdFile": "%s/../d2c/commands/version.py" % (os.path.dirname(__file__)),
                "cmdClass": "VersionCLI"
            },                 
        ]
    },
    # Test
    {
        "name": "Tests",
        "help": "Test functions",
        "envVars":["D2CCLI_TEST"],        
        "commands": [
            {
                "name": ["verify-d2c-model"],
                "cmdFile": "%s/../d2c/test/verify_d2c_model.py" % (os.path.dirname(__file__)),
                "cmdClass": "VerifyD2CModelCLI"
             },                
            {
                "name": ["get-sample-configuration-file"],
                "cmdFile": "%s/../d2c/test/print_sample_test_configuration.py" % (os.path.dirname(__file__)),
                "cmdClass": "PrintSampleTestConfigurationCLI"
            },               
            {
                "name": ["test-application-00"],
                "cmdFile": "%s/../d2c/test/application.py" % (os.path.dirname(__file__)),
                "cmdClass": "FunctionalTest_Application00CLI"
            },    
            {
                "name": ["test-application-01"],
                "cmdFile": "%s/../d2c/test/application.py" % (os.path.dirname(__file__)),
                "cmdClass": "FunctionalTest_Application01CLI"
            },    
            {
                "name": ["test-device-00"],
                "cmdFile": "%s/../d2c/test/device.py" % (os.path.dirname(__file__)),
                "cmdClass": "Tests00_CLI"
            },                             
            {
                "name": ["test-device-01"],
                "cmdFile": "%s/../d2c/test/device.py" % (os.path.dirname(__file__)),
                "cmdClass": "Tests01_CLI"
            },                             
            {
                "name": ["test-device-group-01"],
                "cmdFile": "%s/../d2c/test/device_group.py" % (os.path.dirname(__file__)),
                "cmdClass": "Tests01_CLI"
            },                
            {
                "name": ["test-device-group-02"],
                "cmdFile": "%s/../d2c/test/device_group.py" % (os.path.dirname(__file__)),
                "cmdClass": "Tests02_CLI"
            },                
            {
                "name": ["device-group-test-03"],
                "cmdFile": "%s/../d2c/test/device_group.py" % (os.path.dirname(__file__)),
                "cmdClass": "Tests03_CLI"
            },                   
            {
                "name": ["test-device-group-04"],
                "cmdFile": "%s/../d2c/test/device_group.py" % (os.path.dirname(__file__)),
                "cmdClass": "Tests04_CLI"
            },                   
            {
                "name": ["verify-device-group-consistency"],
                "cmdFile": "%s/../d2c/test/verify_device_group_consistency.py" % (os.path.dirname(__file__)),
                "cmdClass": "VerifyDeviceGroupConsistencyCLI"
            },                   
            {
                "name": ["get-resource-performance-test"],
                "cmdFile": "%s/../d2c/test/perftest_get_resource.py" % (os.path.dirname(__file__)),
                "cmdClass": "PerformanceTest_GetResourceCLI"
            },          
        ]
    },    
]



import json
import logging
import uuid
from datetime import datetime

import requests

import dtiot_d2c.dmo as dmo
from dtiot_d2c.dmo import ApiClient

from . import html_utils as hutils

log = logging.getLogger(__name__)

def _build_timestamp_name(prefix:str):
    ts = datetime.now().strftime('%Y%m%d-%H%M%S')
    return f"{prefix}{ts}"
   
def create_request(api_client:ApiClient, # Client connection definition.
                   origin:str=None,      # Value for X-M2M-Origin header 
                   **kwargs              # Elements of the provisioning data which are placed
                                         # into the dictionary "requestList" of the provisoning 
                                         # request
                  ):     
    '''
    POST {{API_URL}}/device-management-orchestrator/v3/{{TENANT}}/device-provisioning/request HTTP/2
    Accept: application/json
    Content-Type: application/json;ty=28
    X-M2M-Origin: CTestApp
    fullName: true
    X-M2M-RI: 123
    Authorization: Bearer {{ACCESS_TOKEN}}

    {
        "dtiot:noPRt": {
            "cnd": "com.telekom.iot.orchestrator.nodeProvisioningRequest",
            "rn": "myProvisioningRequest01",
            "operation": 1,
            "requestList": [
                {
                    "nodeID" : "urn:gsma:imei:3519...88011",
                    "nodeResourceName" : "sensor001",
                    "ICCID" : "898...20111",
                    "profile" : "SCS-lwM2M",
                    "nodeCredentials": ["psk-id:test@test.com","psk:test123"],
                    "ndvi": {
                        "rn": "deviceInfo2055",
                        "mgd": 1007,
                        "man": "evel-corp",
                        "dty": "world-destructor",
                        "mod": "WD-15",
                        "dlb": "some:label",
                        "dc": "it destroys everything"
                    },
                }
            ]
        }
    }
    '''    
    url = f"{api_client.api_url}/device-provisioning/request"
    hutils.log_url(log.debug, "POST", url)

    headers = {
        "Content-Type"  : f"application/json;ty={dmo.TYs['fcn']}",
        "Accept"        : "application/json",
        "Authorization" : f"Bearer {api_client.bearer_token}",
        "X-M2M-Origin"  : f"C{origin or api_client.origin}",        
        "X-M2M-RI"      : f"{str(uuid.uuid1())}",
        "fullName"      : "true",
    }
    hutils.log_headers(log.debug, headers)

    body = {
        "dtiot:noPRt": {
            "cnd": "com.telekom.iot.orchestrator.nodeProvisioningRequest",
            "rn": _build_timestamp_name("device_provisioning_request_"),
            "operation": dmo.DEVPROV_OPs["create"],
            "requestList": []
        }
    }

    # Build and add the request data to the provisioning request
    requestList = {}
    for (key, value) in kwargs.items():
        requestList[key] = value
    body["dtiot:noPRt"]["requestList"].append(requestList)
        
    hutils.log_body(log.debug, body)
    
    response = requests.post(url=url, headers=headers, json=body)
    hutils.log_response(log.debug, response)    
    
    if not response.ok:
        try:
            log.error(response.content)
        except:
            pass
        response.raise_for_status()
    
    d = json.loads(response.content)

    return d

def get_requests(api_client:ApiClient, # Client connection definition.
                origin:str=None,      # Value for X-M2M-Origin header 
                name:str=None,
                id:str=None,
                select:str=None,
                format:str=None,
                attribute_filters:list=None,
               )->dict:
    return dmo.get_resources(api_client, 
                             origin=origin,
                             resourceId=id,
                             resourceName=name, 
                             resultContentIndocator=dmo.RCIs["attributes-and-children"],
                             select=select,
                             format=format,
                             url_suffix="device-provisioning/request",
                             json_response_filter="m2m:cnt.dtiot:noPRt",
                             attribute_filters=attribute_filters)    
    
def get_responses(api_client:ApiClient, # Client connection definition.
                  origin:str=None,      # Value for X-M2M-Origin header 
                  name:str=None,
                  id:str=None,
                  select:str=None,
                  format:str=None,
                  attribute_filters:list=None,
                 )->dict:
    
    return dmo.get_resources(api_client, 
                             origin=origin,
                             resourceId=id,
                             resourceName=name, 
                             resultContentIndocator=dmo.RCIs["attributes-and-children"],
                             select=select,
                             format=format,
                             url_suffix="device-provisioning/response",
                             json_response_filter="flexContainer", #"m2m:cnt.dtiot:prRRe",
                             attribute_filters=attribute_filters)



import json


def log_url(log_func, operation:str, url:str):
    log_func("")    
    log_func(f"URL: {operation} {url}")

def _maskAuth(headers:dict, header_name:str)->bool:
    if not (auth := headers.get(header_name, None)):
        return False
    elif len(auth) < 30:
        return False
    elif auth.startswith("Bearer "):
        headers[header_name] = "Bearer ***" #f"{auth[0:30]} ..."        
        return True
    else:
        headers[header_name] = "***" #f"{auth[0:30]} ..."        
        return True
    
def log_headers(log_func, headers:dict):
    h = headers.copy()
    #_maskAuth(h, "Authorization")
    #_maskAuth(h, "authorization")
    log_func(f"HEADERS:\n{json.dumps(h, indent=4)}")
    
def log_body(log_func, body:dict):
    if not body:
        body = ""
    log_func(f"BODY:\n{json.dumps(body, indent=4)}")

def log_response(log_func, response):
    log_func(f"RESPONSE CODE: {response.status_code}")
    log_func(f"RESPONSE HEADERS:")
    for header, value in response.headers.items():
        log_func(f"  {header}: {value}")    
    log_func(f"RESPONSE CONTENT: {response.content}")




import json
import sys
import traceback

from dtiot_d2c.d2c import utils as utils
from dtiot_d2c.d2c.utils import color as color

# Log levels
DEBUG = 10
INFO = 20
WARNING=30
ERROR = 40
CRITICAL = 50

def print_ERROR(s:str, **kwargs):
    print(f"{color.RED}{s}{color.END}", **kwargs)
    
def print_OK(s:str = "OK", **kwargs):
    print(f"{color.GREEN}{s}{color.END}", **kwargs)

def print_BOLD(s:str, **kwargs):
    print(f"{color.BOLD}{s}{color.END}", **kwargs)       
    
def load_test_configuration(run_id:str, config_file:str, config_var:str)->dict:
    ### Load configuration
    cfg_mod = utils.importModule("test_configuration", config_file)
    cfg = getattr(cfg_mod, config_var)

    # Replace some config variables
    s = json.dumps(cfg)
    for (var_name, var_value) in [["%RUNID%", run_id]]:
        s = s.replace(var_name, var_value)
    cfg = json.loads(s)
    
    #for (k, v) in cfg.items():
    #    if cfg[k] and type(cfg[k]) == str:
    #        cfg[k] = cfg[k].replace("%RUNID%", run_id)    
            
    return cfg            

def call_has_element_value(d:dict, elem_path:str, test_value):
    b = utils.has_element_value(d, elem_path, test_value)
    #ok = "OK" if b else "NOT OK"
    #print(f"{ok}: has_element_value(node_response, {elem_path}, {test_value})")  
    return b  

def exec_test(label:str, test_func, *args, **kwargs):
    try:
        print(f"{label} ... ", end="", flush=True)
        rv = test_func(*args, **kwargs)
        print_OK(flush=True)
        return rv
    except Exception as ex:
        print_ERROR(f"ERROR: {ex}", flush=True)
        traceback.print_exc()
        return str(ex)




import dtiot_d2c.d2c.version_info as version_info
from dtiot_d2c.d2c.application import Application
from dtiot_d2c.d2c.consts import *
from dtiot_d2c.d2c.device import Device, Message
from dtiot_d2c.d2c.device_group import DeviceGroup
from dtiot_d2c.d2c.onem2m_base import OneM2MBase


def get_version_info()->dict:
    """Returns version if of d2c cli

    Returns:
        dict: {"build_version":"<build version>", "build_date":"<build date>"}
    """

    return {
        "build_version":version_info.build_version,
        "build_date":version_info.build_date
    }



from dtiot_d2c.dmo.api_client import ApiClient, ApiClients, ApiToken
from dtiot_d2c.dmo.consts import *
from dtiot_d2c.dmo.dmo import (add_resource, delete_resource, get_resources,
                               update_resource, update_resource_labels)
from dtiot_d2c.dmo.dmoCLI import _CFG as CLICFG
from dtiot_d2c.dmo.onem2m_utils import (convert_dict_to_onem2m_labels,
                                        convert_onem2m_labels_to_dict,
                                        convert_onem2m_timestamp_to_datetime,
                                        select_from_response_body)


class TimeoutError(Exception):
    def __init__(self, message):
        super().__init__(message)
        


import json
import logging
import sys
import time
from typing import List

import dtiot_d2c.d2c.test.utils as tutils
import dtiot_d2c.dmo as dmo
from dtiot_d2c.d2c import utils as utils
from dtiot_d2c.d2c.application import Application
from dtiot_d2c.d2c.application.consts import *
from dtiot_d2c.d2c.device.consts import *
from dtiot_d2c.d2c.get_response import GetResponse
from dtiot_d2c.d2c.test.testcli_base import TestCase, TestCLI_Base, TestSet
from dtiot_d2c.d2c.utils import color as color
from dtiot_d2c.dmo import ApiClient

log = logging.getLogger(__name__)
log_level = log.getEffectiveLevel() 

def create_application(api: ApiClient, name:str, props:dict):
    def operation():   
        response = Application.create(api,
                                      name=name,
                                      urls=props.get("urls", []),
                                      type=props.get("type", None),
                                      connection_properties=props.get("connection_properties", None)
                                     )
        if not response:
            return f"Application {name} hasn't been created"
        
        if log_level <= tutils.INFO:
            GetResponse(response=response).print()      

    tutils.exec_test(f"Creating ae {name}", operation)
    return None        
        
def verify_application(api:ApiClient, name:str, props:dict):
    '''
    {
        "api": "Nrba-appl-20250404-t002-1",
        "rn": "rba-appl-20250404-t002-1",
        "lbl": [
            "applicationType:WebHook"
        ],
        "poa": [
            "https://dev.scs.iot.telekom.com/scs-callback-dummy"
        ],
        "ty": 2,
        "ri": "67efbea49b1f20af72d58d78",
        "aei": "Crba-appl-20250404-t002-1",
        "pi": "d2c-dev-1",
        "ct": "20250404T111236,691000",
        "lt": "20250404T111236,691000"
    }        
    '''    
    def operation():
        onem2m_dict = dmo.get_resources(api_client=api, resourceName=name)
        if not onem2m_dict:
            raise Exception(f"Application resource {name} couldn't be get after creation.")
        else:
            return onem2m_dict
    onem2m_dict:dict = tutils.exec_test(f"Getting ae {name}", operation)

    elem_path = "rn"
    elem_value = name
    def operation():
        if not tutils.call_has_element_value(onem2m_dict, elem_path, elem_value):
            raise Exception(f"Value of element {elem_path} does not meet required value {elem_value}.")
    tutils.exec_test(f"Verifing {elem_path} for {elem_value}", operation)

    elem_path = "lbl"
    elem_value = f'type:{props.get("type", APPL_TYPE_DEFAULT)}'
    def operation():
        if not tutils.call_has_element_value(onem2m_dict, elem_path, elem_value):
            raise Exception(f"Value of element {elem_path} does not meet required value {elem_value}.")
    tutils.exec_test(f"Verifing {elem_path} for {elem_value}", operation)

    for (key, val) in props.get("connection_properties", {}).items():
        elem_path = "lbl"
        appl_type = props.get("type", APPL_TYPE_DEFAULT)
        prefix = CONN_PROPS_LABEL_PREFIXS.get(appl_type, None)
        elem_value = f"{prefix}{key}:{val}"
        def operation():
            if not tutils.call_has_element_value(onem2m_dict, elem_path, elem_value):
                raise Exception(f"Value of element {elem_path} does not meet required value {elem_value}.")
        tutils.exec_test(f"Verifing {elem_path} for {elem_value}", operation)

    elem_path = "poa"
    elem_value = props.get("urls", None)
    def operation():
        if not tutils.call_has_element_value(onem2m_dict, elem_path, elem_value):
            raise Exception(f"Value of element {elem_path} does not meet required value {elem_value}.")
    tutils.exec_test(f"Verifing {elem_path}: {elem_value}", operation)
    return None    

def delete_application(api:ApiClient, name:str):
    def operation():  
        if (application := Application.get(api, name=name)):
            Application.delete(api, name=name)
    tutils.exec_test(f"Deleting ae {name}", operation)
    return None
        
class TestCase_CreateApplication(TestCase):
    def get_name(self):
        return "Create Application"
        
    def run(self):
        if not (props := self.cfg.get("create_props", None)):
            raise Exception(f"Cannot find element create_props in configuration.")
        return create_application(self.api, self.cfg.get("name"), props)

    def verify(self):
        if not (props := self.cfg.get("create_props", None)):
            raise Exception(f"Cannot find element create_props in configuration.")          
        return verify_application(self.api, self.cfg.get("name"), props)

class TestCase_ModifyApplication(TestCase):
    def get_name(self):
        return "Modify Application"
    
    def run(self):
        name = self.cfg.get("name")

        if not (modify_props := self.cfg.get("modify_props", None)):
            raise Exception(f"Cannot find element modify_props in configuration.")  
        
        def operation():      
            response = Application.update(self.api, 
                                        name=name,
                                        urls=modify_props.get("urls", []),
                                        labels=modify_props.get("labels", None),
                                        connection_properties=modify_props.get("connection_properties", None),
                                        )        
            
            if not response:
                raise Exception(f"Application {name} hasn't been modified")
        
            if log_level <= tutils.INFO:
                GetResponse(response=response).print()      

        tutils.exec_test(f"Modifing ae {name}", operation)
        return None

    def verify(self):
        if not (props := self.cfg.get("modify_props", None)):
            raise Exception(f"Cannot find element modify_props in configuration.")          
        return verify_application(self.api, self.cfg.get("name"), props)

class TestCase_DeleteApplication(TestCase):
    def get_name(self):
        return "Delete Application"
    
    def run(self):
        delete_application(self.api, self.cfg.get("name"))

    def verify(self):    
        name = self.cfg.get("name")        
        def operation():
            if dmo.get_resources(api_client=self.api, resourceName=name):
                raise Exception(f"Delete request didn't delete application {name}.")
        tutils.exec_test(f"Checking if ae {name} has been deleted", operation)

###
# MARK: Tests 00
###    
class Application00Tests(TestSet):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Application Tests 01")
    
    def get_sample_configuration_varname(self)->str:
        return "Application01Cfg"

    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            TestCase_CreateApplication(api=self.api, cfg=cfg),
            TestCase_ModifyApplication(api=self.api, cfg=cfg),
        ]
    
    def test_pre_cleanup(self, cfg:dict)->str:
        delete_application(self.api, cfg.get("name"))

    def test_post_cleanup(self, cfg:dict)->str:
        delete_application(self.api, cfg.get("name"))
    
        
class FunctionalTest_Application00CLI(TestCLI_Base):
    def get1LineHelp(self):
        return "Runs the application test case: create, modify."

    def get_test_set(self, api: ApiClient):
        return Application00Tests(api)


###
# MARK: Tests 01
###    
class Application01Tests(TestSet):
    def __init__(self, api: ApiClient):
        super().__init__(api, "Application Tests 01")
    
    def get_sample_configuration_varname(self)->str:
        return "Application01Cfg"

    def get_test_cases(self, cfg:dict)->List[TestCase]:
        return [
            TestCase_CreateApplication(api=self.api, cfg=cfg),
            TestCase_ModifyApplication(api=self.api, cfg=cfg),
            TestCase_DeleteApplication(api=self.api, cfg=cfg)
        ]
    
    def test_pre_cleanup(self, cfg:dict)->str:
        delete_application(self.api, cfg.get("name"))

    def test_post_cleanup(self, cfg:dict)->str:
        delete_application(self.api, cfg.get("name"))
    
        
class FunctionalTest_Application01CLI(TestCLI_Base):
    def get1LineHelp(self):
        return "Runs the application test case: create, modify, delete application."

    def get_test_set(self, api: ApiClient):
        return Application01Tests(api)




import logging

import dtiot_d2c.cli.CFG as CFG
import dtiot_d2c.cli.utils as utils
from dtiot_d2c.cli.cli_command import CLICommands as clicmds
from dtiot_d2c.cli.cli_command import CmdLineInterface
from dtiot_d2c.cli.config import Configuration
from dtiot_d2c.cli.profileCLI import *

log = logging.getLogger(__name__)

#################################################################
# Create Configuration
#################################################################
class CreateConfigurationCLI(CmdLineInterface):
    def __init__(self):
        super().__init__()

    def get1LineHelp(self):
        return f"Creates a new configuration in a directory or in ~/{CFG.DEFAULT_CFGDIR_NAME}."
    
    def addCmdLineArgs(self, argsParser):

        argsParser.add_argument("--config-directory", metavar='<directory>', dest="directory", 
                                help="Directory in which the new configuration shall be created.")

        argsParser.add_argument("--overwrite", dest="overwrite", action="store_const", const=True,
                                default=False, 
                                help="If set an existing configuration file is overwritten.")

    def main(self, cmdargs=None):
        log.debug("=> CreateConfigurationCLI()")

        # Create the configuration
        config  = Configuration.create(configDir=cmdargs.directory, overwrite=cmdargs.overwrite)

        #  Let the user add a new profile
        profile = NewProfileCLI().main(config=config)

        # Activate profile
        ActivateProfileCLI().main(config=config, profile=profile)

        return config




import importlib
import importlib.util
import json
import logging
import os
import os.path
import sys
from datetime import datetime
from pathlib import Path

log = logging.getLogger(__name__)

class color:
   PURPLE = '\033[95m'
   CYAN = '\033[96m'
   DARKCYAN = '\033[36m'
   BLUE = '\033[94m'
   GREEN = '\033[92m'
   YELLOW = '\033[93m'
   RED = '\033[91m'
   BOLD = '\033[1m'
   UNDERLINE = '\033[4m'
   END = '\033[0m'

import importlib
import sys

if sys.version_info.minor < 12:
    import imp
    def importModule(moduleName, moduleFile):
        """
        Imports a module from source code and returns the exected module
        @param moduleName Name of the module
        @param moduleFile File of the source code
        @return Excecuted module is returned 
        @throws Exception in case of an error
        """
        log.debug("=> importModule(moduleName=%s, moduleFile=%s)" % (moduleName, moduleFile))
        return imp.load_source(moduleName, moduleFile)
else:
    def importModule(moduleName, moduleFile):
        """
        Imports a module from source code and returns the exected module
        @param moduleName Name of the module
        @param moduleFile File of the source code
        @return Excecuted module is returned 
        @throws Exception in case of an error
        """
        log.debug("=> importModule(moduleName=%s, moduleFile=%s)" % (moduleName, moduleFile))
        #return importlib.machinery.SourceFileLoader(moduleName, moduleFile).load_module()
        
        #def import_module(module_name, module_file):
        spec = importlib.util.spec_from_file_location(moduleName, moduleFile)
        module = importlib.util.module_from_spec(spec)
        sys.modules[moduleName] = module
        spec.loader.exec_module(module)
        return module        

def newInstanceFromClass(moduleName, className):
    '''
    Creates an instance of a class from a module and class name string.
    @param moduleName Name of the module
    @param className Name of the class in the module.
    @return Instance of class
    @throws Exception in case of an error.
    '''
    log.debug("=> createClassInstance(moduleName=%s, className=%s)" % (moduleName, className))

    mod = importlib.import_module(moduleName)
    cl = getattr(mod, className, None)

    if cl == None:
        raise Exception("Could not get class object %s from module %s." % (className, moduleName))

    obj = cl()

    if obj == None :
        raise Exception("Could not create instance of class %s from module %s." % (className, moduleName))
    
    log.debug("<= createClassInstance()");
    return obj

def splitString(str, sepChar):
    a = []
    for s in str.split(sepChar):
        a.append(s.strip())
    return a

def resolveVarsInPath(p):
    # Resolve home directory and environment variables
    return os.path.expandvars(p.replace("~", str(Path.home())))

def readJsonFile(filename):
    log.debug("=> read(%s)" % (filename))
    file = open(filename, "r")
    txt = file.read()
    return json.loads(txt)

def getUserInput(title = "", prompt = "", defaultValue=None, requiresInput=False):

    if defaultValue == None:
        prompt = "%s%s: " % (prompt, title)
    else:
        defVal = defaultValue if defaultValue else ""
        prompt = "%s%s [%s]: " % (prompt, title, defVal)

    while True:
        print("%s" % (prompt), end="")
        rv = input()
            
        if rv:
            rv = rv.strip()
        else:
            rv = ""
        
        if len(rv) == 0 and (defaultValue != None and len(str(defaultValue)) > 0):
            rv = defaultValue
        
        if len(str(rv)) > 0 or (len(str(rv)) == 0 and not requiresInput): 
            return rv
               

def backupFile(srcFilepath):

    # Build a unique filename 
    now = datetime.now() 
    date_time = now.strftime("%Y%m%d_%H%M%S")

    count = 0
    while True:
        destFilepath = "%s.%s.%d" % (srcFilepath, date_time, count)
        if not os.path.exists(destFilepath):
            break
        else:
            count = count + 1

    srcFile = open(srcFilepath, "r")
    destFile = open(destFilepath,"w")
    destFile.write(srcFile.read())
    srcFile.close()
    destFile.close()

def isDict(obj):
    if not obj:
        return False
    else:
        return True if obj.__class__ == {}.__class__ else False

def isArray(obj):
    if not obj:
        return False
    else:
        return True if obj.__class__ == [].__class__ else False

def getFilesInDir(dir):
    return [f for f in os.listdir(dir) if os.path.isfile(os.path.join(dir, f))]

def getDirInDir(dir):
    return [f for f in os.listdir(dir) if os.path.isdir(os.path.join(dir, f))]    

def copyFile(fromFile, toFile):
    with open(toFile, 'w') as outFd, open(fromFile, 'r') as inFd:
        outFd.write(inFd.read())

def readJsonFile(filename):
    log.debug("=> readJsonFilie(%s)" % (filename))
    file = open(filename, "r")
    txt = file.read()
    return json.loads(txt)

def writeJsonFile(pyDict, filename):
    log.debug("=> writeJsonFile(pyDict, %s)" % (filename))
    file = open(filename, "w")
    file.write(json.dumps(pyDict, indent=4))
    file.close()


def exec_cmd(cmd, returnAsDict=False, returnAsLinesArray=False):
    cmdIn = os.popen(cmd)

    line = cmdIn.readline()
    s = ""
    while line:
        if line.startswith("b\'"):
            line = line.strip()
            line = line[2:-1] + "\n"
        s = s + line
        line = cmdIn.readline()

    if returnAsDict:
        return json.loads(s)
    elif returnAsLinesArray:
        return splitString(s, "\n")
    else:
        return s

class FileLinesIterator:
    '''
    Iterator class to walk through lines of a text file in the context of for loop.

    Sample code:    

    from file_lines_iter import FileLines

    for line in FileLinesIterator("/Users/rolan/Desktop/WORK/1646599091.json"):
        print(line)
        
    '''
    
    def __init__(self, filepath, stripLines:bool=False, skipEmptyLines:bool=False, skipLinesStartWith:list[str]=[]):
        '''
        Constructor of FileLinesIterator class.
        :param filepath: Path of the file through which lines shall be iterated. If "-" than stdin is read
        :param skipLines: If set all lines are left and right stripped.
        :param skipEmptyLines: If set to true empty lines are not returned.
        :param skipLinesStartWith: Array of single characters. If a line starts with one of the chars it is not returned.
        '''
        self._filepath = filepath
        self._file = None
        self._skipEmptyLines=skipEmptyLines
        self._skipLinesStartWith=skipLinesStartWith
        self._stripLines=stripLines

    def __iter__(self):
        return self
    
    def __next__(self):
        if not self._file:
            if self._filepath=="-":
                self._file = sys.stdin
            else:
                self._file = open(self._filepath, "r")

        while True:
            line = self._file.readline()
            if not line:
                if self._filepath!="-":
                    self._file.close()
                raise StopIteration

            if self._stripLines:
                line=line.strip()
            else:
                line=line.rstrip()
                
            if self._skipEmptyLines and len(line)==0:
                continue

            if len(line)>0 and line[0] in self._skipLinesStartWith:
                continue

            return line

def readLinesFromFile(filepath:str, 
                      stripLines:bool=False, 
                      skipEmptyLines:bool=False, 
                      skipLinesStartWith:list[str]=[]) -> list([str]):
    if not filepath:
        return []
    
    lines = []
    for line in FileLinesIterator(filepath, 
                                  stripLines=stripLines,
                                  skipEmptyLines=skipEmptyLines,
                                  skipLinesStartWith=skipLinesStartWith):
        lines.append(line)
        
    return lines

########################################################################
#MARK: output functions
########################################################################
def print_stdout(s:str, end:str="\n", flush:bool=False):
    sys.stdout.write(f"{s}{end}")

    if flush:
        sys.stdout.flush()    
        
def print_stderr(s:str, end:str="\n", flush:bool=False):
    sys.stderr.write(f"{s}{end}")

    if flush:
        sys.stdout.flush()    


from fastapi import Request, HTTPException

from . import api_utils
import config as CFG
from iam import TokenInfo
from report_job import ReportJob

import logging
log = logging.getLogger(__name__)


def init():
    pass

def handleRequest(token_info:TokenInfo, req:Request, 
                  id:int
                ) -> str:
    api_utils.logRequest(req)

    try:

        report_filepath = ReportJob.download_report_file(id, CFG.JobCfg["downloadDir"])
        return report_filepath

    except Exception as ex:
        log.error(f"Could not download report file from db for job {id}: {ex}.")
        raise HTTPException(status_code=500, detail=f"Internal error.")







#####################################################################
# Copyright (c) 2020 Telekom Deutchland IoT GmbH                    #
# All rights reserved                                               #
#                                                                   #
# Licensed under the MIT license. See LICENSE file in the project   # 
# root for full license information.                                #
#####################################################################

import glob
import importlib
import logging
import os
import os.path
from tkinter import *

import dtiot_d2c.cli.CFG as CFG

log = logging.getLogger(__name__)

class ProfileSelector(Frame):

    def __init__(self, parent, profiles, controller, selectionChangedCallback):
        super().__init__(parent)

        self._profiles = profiles
        self._controller = controller
        
        self._config = None
        self._shownProfiles = None

        self._selectionChangedCallback = selectionChangedCallback
        
        self._selectedProfile = None
        self._selectedProfileVar = None
        self._profileList = None
        self._filterVar = None

        # Build the top area which displays current selected configuration and
        # contains the filter input
        frame = Frame(self)

        self._buildCurrentlySelectedLabel(frame, row=0)
        self._buildFilterInput(frame, row=1)
        
        frame.columnconfigure(1, weight=1)
        frame.pack(side=TOP, expand=False, fill=X)  

        # List of selectable access configurations
        frame = self._buildList(self)
        frame.pack(side=TOP, expand=True, fill=BOTH)

        self._filterVar.trace("w", self._filterProfiles)

        # Show the profiles in the list
        self.updateProfilesList()

    def _buildCurrentlySelectedLabel(self, parent, row):
        
        label = Label(parent, text = "Active:")
        label.grid(row=row, column=0, padx='5', pady='5', sticky='ew')
        
        self._selectedProfileVar = StringVar()
        label = Label(parent, text = "", justify=LEFT, textvariable=self._selectedProfileVar, bg="red", fg="white")
        label.grid(row=row, column=1, padx='5', pady='5', sticky='ew')

    def _buildFilterInput(self, parent, row):
        
        label = Label(parent, text = "Filter:")
        label.grid(row=row, column=0, padx='5', pady='5', sticky='ew')
        
        self._filterVar = StringVar()
        entry = Entry(parent, justify=LEFT, textvariable=self._filterVar)
        entry.grid(row=row, column=1, padx='5', pady='5', sticky='ew')

    def _buildList(self, parent):
        frame = Frame(parent)

        listbox = Listbox(frame, justify=LEFT, width=60, height=30)
        listbox.pack(side=LEFT, expand=True, fill=BOTH)
        self._profileList = listbox

        scrollbar = Scrollbar(frame)
        scrollbar.pack(side = RIGHT, fill = BOTH)

        listbox.bind('<<ListboxSelect>>', self.onSelect)        
        listbox.bind('<Double-Button-1>', self.onDoubleClick)

        listbox.config(yscrollcommand = scrollbar.set)
        scrollbar.config(command = listbox.yview)

        return frame

    def updateProfilesList(self, filter=None, select=None):
        self._profileList.delete(0, END)
        self._shownProfiles = []

        if filter:
            filter = filter.lower()

        if select:
            selectFile = select.getFile()
        else:
            selectFile = None

        idx = 0

        for profile in self._profiles.getProfiles(sorted=os.truncate):
            file = profile.getFile()

            if not file:
                log.warn("File of profile not defined.")
                continue

            # Cut off leading directory
            label = os.path.basename(file)

            # Cut off the ".profile" file extension
            idx = label.find(".%s" % (CFG.PROFILE_FILE_EXT))
            if idx > -1:
                label = label[0:idx]

            if filter and len(filter) > 0:
                if label.lower().find(filter) == -1:
                    continue

            self._profileList.insert("end", label)
            self._shownProfiles.append(profile)
            
            if selectFile and file == selectFile:
                self._profileList.select_set(idx)

            idx = idx + 1

    def _filterProfiles(self, *args):
        filter = self._filterVar.get()
        filter = filter.strip() if filter else None

        self.updateProfilesList(filter)
       
    def onSelect(self, evt):
        log.info("=> onSelect()")
        w = evt.widget
        index = int(w.curselection()[0])
        value = w.get(index)
       
        self._selectionChangedCallback(self._shownProfiles[index])

    def showActivatedProfile(self, profile):
        if profile:
            file = os.path.basename(profile.getFile())
            idx = file.find(".%s" % (CFG.PROFILE_FILE_EXT))
            if idx > -1:
                file = file[:idx]
            self._selectedProfileVar.set(file)
        else:
            self._selectedProfileVar.set("")

    def onDoubleClick(self, evt):
        pass
        '''
        w = evt.widget
        index = int(w.curselection()[0])
        value = w.get(index)
        cmd = "gnome-terminal"
        log.debug("Exec %s" % (cmd))
        os.system(cmd)
        '''

    '''
    def setConfig(self, config):
        self._config = config
        self.updateProfilesList()
    '''



import logging
import os
from argparse import ArgumentParser

import dtiot_d2c.cli.CFG as CFG
import dtiot_d2c.cli.utils as utils
from dtiot_d2c.cli.config import Configuration
from dtiot_d2c.cli.profile import Profile, Profiles

log = logging.getLogger(__name__)


class CmdLineInterface:
    def __init__(self, flags=0):
        self._flags = flags
        self._name = "NoName"
        self._aliasNames = []
        self._preDefinedCmdArgs = {}
        self._hideCmdArgs = []
        self._1LineHelp = None

    def getName(self):
        return self._name

    def setName(self, name):
        self._name = name

    def getAliasNames(self):
        return self._aliasNames

    def setAliasNames(self, aliasNames):
        self._aliasNames = aliasNames

    def setPreDefinedCmdArgs(self, preDefinedCmdArgs):
        self._preDefinedCmdArgs = {}
        if preDefinedCmdArgs:
            for pdca in preDefinedCmdArgs:
                self._preDefinedCmdArgs[pdca["name"]] = pdca

    def addPreDefinedCmdArg(self, name, value):
        self._preDefinedCmdArgs[name] = value

    def hasPreDefinedCmdArg(self, name):
        return True if name in self._preDefinedCmdArgs.keys() else None

    def getPreDefinedCmdArg(self, name):
        return self._preDefinedCmdArgs[name] if name in self._preDefinedCmdArgs.keys() else None

    def getPreDefinedCmdArgs(self):
        return self._preDefinedCmdArgs

    def setHideCmdArgs(self, hideCmdArgs):
        self._hideCmdArgs = hideCmdArgs if hideCmdArgs else []

    def hasHideCmdArg(self, findPattern):
        for c in self._hideCmdArgs:
            if c.find(findPattern) > -1:
                return True

    def get1LineHelp(self):
        return "No 1-line-help available."

    def overwrite1LineHelp(self, txt):
        self._1LineHelp = txt

    def callGet1LineHelp(self):
        if self._1LineHelp:
            return self._1LineHelp
        else:
            return self.get1LineHelp()

    def addCmdLineArgs(self, argsParser):
        pass

    def isFlagSet(self, flag):
        return True if (self._flags & flag) > 0 else False

    def main(self, cmdargs=None):
        raise Exception("Method not implemented.")

    def callMain(self, cmdargs=None):
        # If command line args have been passed, check if any pre-defined args
        # have been configured. If yes, add the to cmdargs.
        if cmdargs:
            for key in self._preDefinedCmdArgs:
                pdca = self._preDefinedCmdArgs[key]
                name = pdca["name"]
                value = pdca["value"]
                dest = pdca["dest"] if "dest" in pdca.keys() else None

                if not dest:
                    dest = name.strip("-_").replace("-", "_")

                # Add the pre-defined command argument
                if not hasattr(cmdargs, dest):
                    setattr(cmdargs, dest, value)

        # Call the implementation's main function
        self.main(cmdargs=cmdargs)

    def addCmdLineArgsFromTemplate(self, args_templates:dict, args_parser:ArgumentParser, arg_name:str, *args, **overwrite_params)->bool:
        '''
        Adds a command line option from the configuration of command line options from a template dictionary.
        The template dictionary should have the following form:
        
        {
            "-lbl" : {"metavar":"{key:value, key:value, ...}", 
                    "dest":"labels", 
                    "required":False,
                    "default":None,
                    "help":"Labels defined by a JSON dictionary of key/value pairs."},
            "-o": ["--origin", 
                {"metavar":"<string>", 
                    "dest":"origin", 
                    "required": False,
                    "default":None,
                    "help":"X-M2M-ORIGIN heder."}
            ],
        }
        '''
        cfg = args_templates.get(arg_name, None)
        if not cfg:
            raise Exception(f"No command line argument configuration {arg_name} available.")
        elif type(cfg) not in [dict, list]:
            raise Exception(f"Invalid command line argument configuration for {arg_name}.")

        arg_name2:str = None
        if type(cfg) == list and len(cfg) >= 2:
            arg_name2 = cfg[0]            
            cfg = cfg[1]
        elif type(cfg) == list and len(cfg) == 1:
            cfg = cfg[0]

        _cfg = cfg.copy()
        
        for (k, v) in overwrite_params.items():
            _cfg[k] = v

        if arg_name2:
            args_parser.add_argument(arg_name, arg_name2, **_cfg)
        else:
            args_parser.add_argument(arg_name, **_cfg)

        return True  

class ConfigCmdLineInterface(CmdLineInterface):
    def __init__(self):
        super().__init__()
        self._config:Configuration = None

    def getConfig(self)->Configuration:
        return self._config

    @property  
    def config(self)->Configuration:
        return self._config
    
    @config.setter
    def config(self, config:Configuration):
        self._config = config

class ProfileCmdLineInterface(ConfigCmdLineInterface):
    def __init__(self):
        super().__init__()
        self._profile:Profile = None

    def getProfile(self)->Profile:
        return self._profile
    
    @property  
    def profile(self)->Profile:
        return self._profile
    
    @profile.setter
    def profile(self, profile:Profile):
        self._profile = profile
    
class ApiConnCmdLineInterface(ProfileCmdLineInterface):
    def __init__(self):
        super().__init__()
        self.apiConn = None

class _CLICommands():
    def __init__(self):
        self._cmdgrps = {}
        self._commands = {}

    def addCommandGroup(self, name, help):
        self._cmdgrps[name] = {
            "name": name,
            "help": help,
            "commands": []
        }

    def getCommandGroupNames(self):
        return self._cmdgrps.keys()

    def getCommandGroup(self, name):
        return self._cmdgrps[name]

    def addCommand(self, grp, cmd):
        self._cmdgrps[grp]["commands"].append(cmd)
        self._commands[cmd.getName()] = cmd

        # Add the alias names for  the command to the index
        for ca in cmd.getAliasNames():
            self._commands[ca] = cmd

    def getCommandNames(self):
        names = []
        for key in self._commands.keys():
            cmd = self._commands[key]
            names.append(cmd.getName())
        return names

    def getCommands(self):
        commands = []
        for key in self._commands.keys():
            commands.append(self._commands[key])
        return commands

    def getCommand(self, name):
        for key in self._commands.keys():
            cmd = self._commands[key]
            if name == cmd.getName() or name in cmd.getAliasNames():
                return cmd
        return None

# Create the singelton instance to manage registered cli commands
CLICommands = None

def init():
    log.debug("Loading CLI commands ...")
    for cg in CFG.CLICommands:
        grpname = cg["name"]
        grphelp = cg["help"]

        # If environment variable names are configured to activate
        # the command group check if the environment variable is set.
        # If not, skip this command group because it isn't activated.
        if (env_var_names := cg.get("envVars", [])):
            skip_cg = True
            for env_var_name in env_var_names:
                if (v := os.getenv(env_var_name, None)):
                    if str(v).lower() in ["1", "true", "yes"]:
                        skip_cg = False
                        break
            # The command group hasn't been switched on by environment varialbe
            if skip_cg:
                continue

        log.debug("  group name: %s" % (grpname))
        log.debug("  group help: %s" % (grphelp))

        CLICommands.addCommandGroup(grpname, grphelp)

        for c in cg["commands"]:
            cmdName = c["name"]
            log.debug("    name:  %s" % (str(cmdName)))
            log.debug("    file:  %s" % (c["cmdFile"]))
            log.debug("    class: %s" % (c["cmdClass"]))

            preDefinedCmdArgs = c["preDefinedCmdArgs"] if "preDefinedCmdArgs" in c.keys() else None
            log.debug("    preDefinedCmdArgs: %s" % (str(preDefinedCmdArgs)))

            hideCmdArgs = c["hideCmdArgs"] if "hideCmdArgs" in c.keys() else None
            log.debug("    hideCmdArgs: %s" % (str(hideCmdArgs)))

            help = c["1LineHelp"] if "1LineHelp" in c.keys() else None
            log.debug("    help: %s" % (str(help)))

            cmdAliasNames = []
            if utils.isArray(cmdName):
                cmdAliasNames = cmdName[1:]
                cmdName = cmdName[0]

            modName = "_Command_%s" % (cmdName)
            mod = utils.importModule(modName, c["cmdFile"])
            cmd = utils.newInstanceFromClass(modName, c["cmdClass"])
            cmd.setName(cmdName)
            cmd.setAliasNames(cmdAliasNames)
            cmd.setPreDefinedCmdArgs(preDefinedCmdArgs)
            cmd.setHideCmdArgs(hideCmdArgs)
            cmd.overwrite1LineHelp(help)

            CLICommands.addCommand(grpname, cmd)


# Initialize registered cli commands
if not CLICommands:
    CLICommands = _CLICommands()
    init()



import json
import logging
import time
import uuid
from datetime import datetime
from typing import List

import requests

from dtiot_d2c.dmo import ApiClient
from dtiot_d2c.dmo import html_utils as hutils
from dtiot_d2c.dmo import onem2m_utils as onem2m_utils
from dtiot_d2c.dmo import utils as utils
from dtiot_d2c.dmo.consts import *

log = logging.getLogger(__name__)

DMO_BASEPATH="api-gw/device-management-orchestrator"

################################################################################
# General Resource related Functions
################################################################################
def _add_labels_to_request_body(dest:dict, **kwargs)->list([str]):
    '''
    If the "labels" parameter is in the kwargs than it is translated from a dictionary
    into an array of strings "<key>:<value>" and added to the element "lbl" of the 
    destonation dictionary.
    '''
    labels = kwargs.get("labels", None)
    if labels:
        if type(labels) == str:
            labels = json.loads(labels)                
            
        a = []
        for (k, v) in labels.items():
            a.append(f"{k}:{v}")            
        if len(a) > 0:
            dest["lbl"] = a
    return labels

@utils.profile(log_func=log.info, log_prefix="dmo-profile")
def _get_resources(api_client:ApiClient,                # Client connection definition.
                   origin:str=None,                     # Value for X-M2M-Origin header 
                   resourceType:str=None,               # Resource type of the objects to get. Supported types are: TYs (see above).
                   resourceId:str=None,                 # Id of the resource                   
                   resourceName:str=None,               # Name of a resource which details shall be get. 
                                                        # If not defined the root CSEBase is used.
                   resultContentIndocator:str=None,     # Result content indicator. Supported types are RCIs (see above).
                   limit:int=1000000000,                # Max number of objects to return.
                   offset:int=0,
                   url_suffix:str=None,                 # Path suffix which is appended to the url path.
                   attribute_filters:list=None,
                   json_response_filter:str=None,    
                   use_unstructured_path:bool=False     # If set to True the unstructured path is used
                  ) -> dict:
    '''
    Gets objects of a specific resource type.
    '''
    log.debug("=> _get_resources()")
    log.debug(f"   origin:               {origin}")    
    log.debug(f"   rt:                   {resourceType}")    
    log.debug(f"   ri:                   {resourceId}")    
    log.debug(f"   rn:                   {resourceName}")    
    log.debug(f"   rci:                  {resultContentIndocator}")    
    log.debug(f"   limit:                {limit}")    
    log.debug(f"   offset:               {offset}")    
    log.debug(f"   url_suffix:           {url_suffix}")    
    log.debug(f"   attribute_filters:    {attribute_filters}")    
    log.debug(f"   json_response_filter: {limit}")    
    log.debug(f"   use_unstructured_path: {use_unstructured_path}")    
    log.debug("")    

    if resourceId or use_unstructured_path:
        url = f"{api_client.api_url_unstructured}"
    else:
        url = f"{api_client.api_url}"
        
    if url_suffix:
        url += f"/{url_suffix}"
        
    if resourceId:
        url += f"/{resourceId}"
    elif resourceName:
        url += f"/{resourceName}"

    rcn = RCIs.get(resultContentIndocator, 4)
    url += f"?rcn={rcn}"

    #if not resourceName and not resourceId:
    #    url += f"&limit={limit}&offset={offset}"
    if limit > 0:
        url += f"&limit={limit}&offset={offset}"
        
    if attribute_filters:
        url += f"&{'&'.join(attribute_filters)}"

    #url += f'&fwr={{"cra":"20250610T085823"}}'
    #/{resourcePath}?fwr={"cra":"20230101T000000"}

    ty = None
    # We only deal with the resource type if the no resource name is defined.
    # If a resource name is defined we force the resource type to be None
    if resourceType:
        # ty = TYs.get(resourceType, None)
        # if not ty:
        #     raise Exception(f"Unknown resource type {resourceType}")
        # url += f"&ty={ty}"

        if not resourceName:
            ty = TYs.get(resourceType, None)
            if not ty:
                raise Exception(f"Unknown resource type {resourceType}")
            url += f"&ty={ty}"
        else:
            #log.warning(f"Resource type {resourceType} ignored because resource name defined.")
            resourceType = None


    #/nodes?fc={"createdAfter":"2023-10-01T00:00:00Z"} HTTP
    #url += f'&"creationTime":"2025-06-06T17:11:28"'
    hutils.log_url(log.debug, "GET", url)

    headers = {
        "Content-Type"  : "application/json",
        "Accept"        : "application/json",
        "Authorization" : f"Bearer {api_client.bearer_token}",
        "X-M2M-Origin"  : f"C{origin or api_client.origin}",        
        "X-M2M-RI"      : f"{str(uuid.uuid1())}"
    }
    hutils.log_headers(log.debug, headers)

    body = None
    hutils.log_body(log.debug, body)
    
    response = requests.get(url=url, headers=headers, json=body)
    hutils.log_response(log.debug, response)    

    if not response.ok:
        if response.status_code == 404:
            #log.error(response.content)
            return None
        else:
            try:
                log.error(response.content)
            except:
                pass
            response.raise_for_status()
    
    d = json.loads(response.content)
    #log.debug(json.dumps(d, indent=4))

    if json_response_filter:
        if not resourceName:
            d = utils.getValueFromDict(d, json_response_filter, {})
        else:
            d = utils.getValueFromDict(d, json_response_filter, {})
    
    # If a named resource shall be get
    if resourceName or resourceId:
        if len(d.keys()) == 1:
            for (k, o) in d.items():
                return o
        else:
            return d    

    jsonFilter = TYs_JSON_FILTERS.get(resourceType, None)
    if jsonFilter:
        return utils.getValueFromDict(d, jsonFilter, {})
    
    return d
            
    # if resourceType == 'cb':
    #     return d.get("m2m:cb", {})
    # elif resourceType:
    #     return d.get("m2m:cb", {}).get(f"m2m:{resourceType}",[])
    # else:
    #     if len(d.keys()) == 1:
    #         for (k, o) in d.items():
    #             return o
    #     else:
    #         return d    
        
def get_resources(api_client:ApiClient,                # Client connection definition.
                  origin:str=None,                     # Value for X-M2M-Origin header 
                  resourceType:str=None,               # Resource type of the objects to get. Supported types are: TYs (see above).
                  resourceId:str=None,                 # Id of the resource
                  resourceName:str=None,               # Name of a resource which details shall be get. 
                                                       # If not defined the root CSEBase is used.
                  resultContentIndocator:str=None,     # Result content indicator. Supported types are RCIs (see above).
                  limit:int=1000000000,                # Max number of objects to return.
                  offset:int=0,
                  select:str=None,                     # Comma separated list of element names which shall be selected.
                  format:str="CSV",                    # Output format. Possible values are CSV and JSON
                  url_suffix:str=None,                 # Path suffix which is appended to the url path.        
                  json_response_filter:str=None,     
                  attribute_filters:list=None,   
                  use_unstructured_path:bool=False     # If set to True the unstructured path is used
                 ) -> dict:        

    log.debug("=> get_resources()")
    log.debug(f"   origin:                {origin}")    
    log.debug(f"   rt:                    {resourceType}")    
    log.debug(f"   ri:                    {resourceId}")    
    log.debug(f"   rn:                    {resourceName}")    
    log.debug(f"   rci:                   {resultContentIndocator}")    
    log.debug(f"   limit:                 {limit}")    
    log.debug(f"   offset:                {offset}")    
    log.debug(f"   url_suffix:            {url_suffix}")    
    log.debug(f"   attribute_filters:     {attribute_filters}")    
    log.debug(f"   json_response_filter:  {limit}")    
    log.debug(f"   use_unstructured_path: {use_unstructured_path}")    
    
    log.debug("")    

    o = _get_resources(api_client=api_client, 
                       origin=origin,
                       resourceType=resourceType, 
                       resourceId=resourceId,
                       resourceName=resourceName,
                       resultContentIndocator=resultContentIndocator, 
                       limit=limit, 
                       offset=offset,
                       url_suffix=url_suffix,
                       json_response_filter=json_response_filter,
                       attribute_filters=attribute_filters,
                       use_unstructured_path=use_unstructured_path)
    if not select:
        return o

    selectFields = utils.splitString(select, ",")

    ret = []
    if type(o) != list:
        o = [o]

    if not format:
        format = "CSV"
        
    if format.upper() == "CSV":
        # for i in o:
        #     a = []
        #     for select_field in selectFields:
        #         v = utils.get_value_from_dict_3(i, select_field, default="", sepChar=".")
        #         a.append(str(v))
        #     ret.append(a)
        # return ret
        for i in o:
            a = []
            for s in selectFields:
                if s in i.keys():
                    v=i[s]
                    if type(v) in [dict, list, tuple]:
                        v = json.dumps(v)
                    else:
                        v = str(v)
                    a.append(v)
            ret.append(a)
        return ret
        # for i in o:
        #     a = []
        #     for s in selectFields:
        #         if s in i.keys():
        #             a.append(str(i[s]))
        #     ret.append(a)
        # return ret

    elif format.upper() == "JSON":
        for i in o:
            if type(i) != dict:
                continue
            d = {}
            for s in selectFields:
                if s in i.keys():
                    d[s] = i[s]
            ret.append(d)
            
        if resourceName and ret:
            return ret[0]
        else:
            return ret            
    else:
        raise Exception(f"Unsupported format {format}. JSON or CSV required.") 

@utils.profile(log_func=log.info, log_prefix="dmo-profile")
def add_resource(api_client:ApiClient, 
                 origin:str=None,             # Value for X-M2M-Origin header                  
                 r_type:str=None,             # resource type
                 r_name:str=None,             # resource name
                 onem2m_body:dict=None,       # Body of the http request
                 **kwargs             
                ) -> dict:
    '''
    Adds a resource object to the tenant or another resource.
    '''
    url = f"{api_client.api_url}"
    if r_name:
        url += f"/{r_name}"

    hutils.log_url(log.debug, "POST", url)        

    headers = {
        "Content-Type"  : f"application/json;ty={TYs[r_type]}",
        "Accept"        : "application/json",
        "Authorization" : f"Bearer {api_client.bearer_token}",
        "X-M2M-Origin"  : f"C{origin or api_client.origin}",        
        "X-M2M-RI"      : f"{str(uuid.uuid1())}"
    }
    hutils.log_headers(log.debug, headers)            

    log.debug(f"BODY: {onem2m_body}")

    # if len(resourceNames) == 1:
    #     headers["X-M2M-Origin"] = f"C{apiClient.client_rn}"
    # else:
    #     headers["X-M2M-Origin"] = "/".join(resourceNames[0:-1])
    #headers["X-M2M-Origin"] = f"C{apiClient.client_rn}"
    
    response = requests.post(url=url, headers=headers, json=onem2m_body)   
    hutils.log_response(log.debug, response)    
    
    if not response.ok:
        try:
            log.error(response.content)
        except:
            pass
        response.raise_for_status()
    
    d = json.loads(response.content)

    return d

@utils.profile(log_func=log.info, log_prefix="dmo-profile")
def delete_resource(api_client:ApiClient,  
                    origin:str=None,       # Value for X-M2M-Origin header 
                    name:str=None,          # Name of the resource object to delete.
                    id:str=None
                   ):
    if id:
        url = f"{api_client.api_url_unstructured}/{id}"
    else:
        url = f"{api_client.api_url}/{name}"

    hutils.log_url(log.debug, "DELETE", url)        

    headers = {
        "Content-Type"  : f"application/json",
        "Accept"        : "application/json",
        "Authorization" : f"Bearer {api_client.bearer_token}",
        "X-M2M-Origin"  : f"C{origin or api_client.origin}",        
        "X-M2M-RI"      : f"{str(uuid.uuid1())}"
    }
    hutils.log_headers(log.debug, headers)            

    response = requests.delete(url=url, headers=headers)
    hutils.log_response(log.debug, response)    
    
    if not response.ok:
        try:
            log.error(response.content)
        except:
            pass
        response.raise_for_status()

    return None

@utils.profile(log_func=log.info, log_prefix="dmo-profile")
def update_resource(api_client:ApiClient,   # Client connection definition.
                    origin:str,             # Value for X-M2M-Origin header                     
                    name:str=None,          # Name of the resource object to update.
                    id:str=None,
                    onem2m_body:dict=None,
                  ) -> dict:
    if id:
        url = f"{api_client.api_url_unstructured}/{id}"
    else:
        url = f"{api_client.api_url}/{name}"

    hutils.log_url(log.debug, "PUT", url)        

    headers = {
        "Content-Type"  : f"application/json",
        "Accept"        : "application/json",
        "Authorization" : f"Bearer {api_client.bearer_token}",
        "X-M2M-Origin"  : f"C{origin or api_client.origin}",        
        "X-M2M-RI"      : f"{str(uuid.uuid1())}"
    }
    hutils.log_headers(log.debug, headers)            

    hutils.log_body(log.debug, onem2m_body)

    response = requests.put(url=url, headers=headers, json=onem2m_body)
    hutils.log_response(log.debug, response)    
    
    if not response.ok:
        try:
            log.error(response.content)
        except:
            pass
        response.raise_for_status()
    
    d = json.loads(response.content)

    return d

@utils.profile(log_func=log.info, log_prefix="dmo-profile")
def update_resource_labels(api_client:ApiClient,   # Client connection definition.
                           origin:str,             # Value for X-M2M-Origin header   
                           r_type:str,             # Type of the resource object
                           r_name:str,             # Name of the resource object
                           labels:dict,            # Labels dictionary
                          ) -> dict:
    '''
    Updates the labels of a resource object.
    '''
    url = f"{api_client.api_url}/{r_name}?rcn={RCIs['return-all']}"
    hutils.log_url(log.debug, "PUT", url)        

    headers = {
        "Content-Type"  : f"application/json",
        "Accept"        : "application/json",
        "Authorization" : f"Bearer {api_client.bearer_token}",
        "X-M2M-Origin"  : f"C{origin or api_client.origin}",        
        "X-M2M-RI"      : f"{str(uuid.uuid1())}"
    }
    hutils.log_headers(log.debug, headers)            

    body = {
        f"m2m:{r_type}": {
            "lbl":onem2m_utils.convert_dict_to_onem2m_labels(labels)
        }
    }
    hutils.log_body(log.debug, body)

    response = requests.put(url=url, headers=headers, json=body)
    hutils.log_response(log.debug, response)    
    
    if not response.ok:
        try:
            log.error(response.content)
        except:
            pass
        response.raise_for_status()
    
    d = json.loads(response.content)

    return d

################################################################################
################################################################################
################################################################################
################################################################################

def old__getResources(apiClient:ApiClient,                 # Client connection definition.
                  resourceType:str=None,               # Resource type of the objects to get. Supported types are: TYs (see above).
                  resourceName:str=None,               # Name of a resource which details shall be get. 
                                                       # If not defined the root CSEBase is used.
                  resultContentIndocator:str=None,     # Result content indicator. Supported types are RCIs (see above).
                  limit:int=1000000000,                # Max number of objects to return.
                  origin:str=None                      # Originitor object of this request. If not defined the default from 
                                                       # API client object is used.
                  ) -> dict:
    '''
    Gets objects of a specific resource type.
    '''

    ###
    # Build the URL
    url = f"https://{apiClient.host}/{DMO_BASEPATH}/{apiClient.api_version}/{apiClient.tenant_name}"
    
    if resourceName:
        url += f"/{resourceName}"

    rcn = RCIs.get(resultContentIndocator, 4)
    url += f"?rcn={rcn}&limit={limit}"

    ty = None
    # We only deal with the resource type if the no resource name is defined.
    # If a resource name is defined we force the resource type to be None
    if resourceType:
        if not resourceName:
            ty = TYs.get(resourceType, None)
            if not ty:
                raise Exception(f"Unknown resource type {resourceType}")
            url += f"?&ty={ty}"
        else:
            log.warn(f"Resource type {resourceType} ignored because resource name defined.")
            resourceType = None

    log.debug(f"POST {url}")

    headers = {
        "Content-Type"  : "application/json",
        "Accept"        : "application/json",
        "Authorization" : f"Bearer {apiClient.bearer_token}",
        "X-M2M-Origin"  : f"C{apiClient.client_rn}" if not origin else origin,
        "X-M2M-RI"      : f"dmo_{str(time.time())}"
    }
    log.debug(f"HEADER: {headers}")        

    body = None
    hutils.log_body(log.debug, body)
    
    response = requests.get(url=url, headers=headers, json=body)
    
    if not response.ok:
        try:
            log.error(response.content)
        except:
            pass
        
        response.raise_for_status()
    
    d = json.loads(response.content)
    log.debug(json.dumps(d, indent=4))
    
    # If a named resource shall be get
    if resourceName:
        if len(d.keys()) == 1:
            for (k, o) in d.items():
                return o
        else:
            return d    
        
    if resourceType == 'cb':
        return d.get("m2m:cb", {})
    elif resourceType:
        return d.get("m2m:cb", {}).get(f"m2m:{resourceType}",[])
    else:
        if len(d.keys()) == 1:
            for (k, o) in d.items():
                return o
        else:
            return d    
        
def old_getResources(apiClient:ApiClient,                  # Client connection definition.
                 resourceType:str,                     # Resource type of the objects to get. Supported types are: TYs (see above).
                 resourceName:str=None,                # Name of a resource which details shall be get. 
                                                       # If not defined the root CSEBase is used.
                 resultContentIndocator:str=None,      # Result content indicator. Supported types are RCIs (see above).
                 limit:int=1000000000,                 # Max number of objects to return.
                 origin:str=None,                      # Originitor object of this request. If not defined the default from
                                                       # API client object is used.
                 select:str=None,                      # Comma separated list of element names which shall be selected.
                 format:str="CSV"                      # Output format. Possible values are CSV and JSON
                 ) -> dict:        

    o = old__getResources(apiClient=apiClient, 
                      resourceType=resourceType, 
                      resourceName=resourceName,
                      resultContentIndocator=resultContentIndocator, 
                      limit=limit, origin=origin)

    if not select:
        return o

    selectFields = utils.splitString(select, ",")

    ret = []
    if type(o) != list:
        o = [o]

    if not format:
        format = "CSV"
        
    if format.upper() == "CSV":

        for i in o:
            a = []
            for s in selectFields:
                if s in i.keys():
                    a.append(str(i[s]))
            ret.append(a)

    elif format.upper() == "JSON":
        for i in o:
            if type(i) != dict:
                continue
            d = {}
            for s in selectFields:
                if s in i.keys():
                    d[s] = i[s]
            ret.append(d)
    else:
        raise Exception(f"Unsupported format {format}. JSON or CSV required.") 
    
    return ret







def addMessageContainer(apiClient:ApiClient,     # Client connection definition.
                        resourceName:str,        # Name of the object to which the container shall be added.
                        containerName:str,       # Name of the container.
                        maxByteSize:int=10000,   # Max. number of bytes in a message.
                        maxNrOfInstaces:int=10,  # Max. number of stored messages.
                        **kwargs                 # "labels":dict - Dictionary of labels. Labels are translated into an 
                                                 #                 array of strings of the form "<key>:<value>"
                       )->dict:
    '''
    Adds a container of uplink messages to a resource object.
    '''
    url = f"https://{apiClient.host}/{DMO_BASEPATH}/{apiClient.api_version}/{apiClient.tenant_name}/{resourceName}"
    log.debug(f"PUT {url}")

    headers = {
        "Content-Type"  : f"application/json;ty=3",
        "Accept"        : "application/json",
        "Authorization" : f"Bearer {apiClient.bearer_token}",
        "X-M2M-Origin"  : f"C{resourceName}", #"X-M2M-Origin"  : f"C{apiClient.client_rn}",
        "X-M2M-RI"      : f"dmo_{str(time.time())}"
    }
    log.debug(f"HEADER: {headers}")        

    # Build the body
    body = {
        "container": {
            "maxByteSize": maxByteSize,
            "maxNrOfInstances": maxNrOfInstaces,
            "resourceName": containerName
        }
    }
    
    # labels
    _add_labels_to_request_body(body["container"], **kwargs)    

    hutils.log_body(log.debug, body)

    response = requests.post(url=url, headers=headers, json=body)
    
    if not response.ok:
        try:
            log.error(response.content)
        except:
            pass
        response.raise_for_status()
    
    d = json.loads(response.content)

    return d

def addSubscription(apiClient:ApiClient,     # Client connection definition.
                    resourceName:str,        # Name of the object to which the subscription shall be added.
                    subscriptionName:str,    # Name of the subscription
                    notificationUrl:str,     # URL to which the events shall be forwarded.
                    notificationContentType:str="all", # Content type of the notification. Siehe NCTs oben
                    eventNotificationCriterias:list([str])=["u","d","cc", "dc"], # Event notififcation criterias
                    **kwargs                 # "labels":dict - Dictionary of labels. Labels are translated into an 
                                             #                 array of strings of the form "<key>:<value>"
                                             # "pendingNotification":str - Defines how to handle pending notifiations.
                                             #                             possible valures are defined in PNs.
                   ) -> dict:
    '''
    Adds a subscription to a resource object.
    '''
    url = f"https://{apiClient.host}/{DMO_BASEPATH}/{apiClient.api_version}/{apiClient.tenant_name}/{resourceName}"
    log.debug(f"PUT {url}")

    headers = {
        "Content-Type"  : f"application/json;ty=23",
        "Accept"        : "application/json",
        "Authorization" : f"Bearer {apiClient.bearer_token}",
        "X-M2M-Origin"  : f"C{apiClient.client_rn}", #f"C{resourceName}", #"X-M2M-Origin"  : 
        "X-M2M-RI"      : f"dmo_{str(time.time())}"
    }
    log.debug(f"HEADER: {headers}")        

    # Build the list of event notification criterias
    encs = []
    if eventNotificationCriterias:
        for s in eventNotificationCriterias:
            encs.append(ENCs.get(s))            

    ###
    # Build the body
    body = {
        "subscription": {
            "rn":  subscriptionName,
            "nu":  [notificationUrl],
            "nct": NCTs.get(notificationContentType),
            "enc": {
                "net": encs
            }
        }
    }

    # pendingNotification    
    pn = kwargs.get("pendingNotification", None)
    if pn:
        body["subscription"]["pn"] = PNs.get(pn)

    # labels
    _add_labels_to_request_body(body["subscription"], **kwargs)

    hutils.log_body(log.debug, body)

    ###
    # Post the request 
    response = requests.post(url=url, headers=headers, json=body)
    
    if not response.ok:
        try:
            log.error(response.content)
        except:
            pass
        response.raise_for_status()
    
    d = json.loads(response.content)

    return d






from enum import Enum

from pydantic import BaseModel

import dtiot_d2c.d2c.consts as d2c_consts


class DEVICE_PROFILES:
    SCS_LWM2M:str = "SCS-LwM2M"
    
class MessageStore(BaseModel):
    rn:str       # OneM2M name
    label:str    # Label to be used in logging

VALID_NAME_CHARS = r'^[0-9A-Za-z_-]+$'
    
INBOUND_DOWNLINK_MESSAGE_STORE  = MessageStore(rn="outgoing-msg", label="Downlink Inbound")
OUTBOUND_DOWNLINK_MESSAGE_STORE = MessageStore(rn="sent-msg", label="Downlink Outbound")
INBOUND_UPLINK_MESSAGE_STORE    = MessageStore(rn="received-msg", label="Uplink Inbound")
OUTBOUND_UPLINK_MESSAGE_STORE   = MessageStore(rn="sent-received-msg", label="Uplink Outbound")

UPLINK_PROPERTY_PREFIX = "UplinkProperty_"
DEVICE_SYSTEM_LABELS = ["profile", "ICCID", d2c_consts.D2C_SYSTEM_VERSION_LABEL]
DEVICE_INFO_RN = "device-info"
DEVICE_INFO_MGD = 1007

# Sleep seconds it is waited after the device credentials have been deleted to create new ones
DELETE_CREDENTIALS_SLEEP_SECS = 1





from fastapi import Request, HTTPException

from . import api_utils
import config as CFG
from iam import TokenInfo
from report_job import ReportJob
from reports.device_report import DeviceReportJob, DeviceReport

import logging
log = logging.getLogger(__name__)


def init():
    pass

def handleRequest(token_info:TokenInfo, req:Request, 
                  tenant_id:int, 
                  when:str=None, 
                  month_of_year:str=None, 
                  from_date:str=None, 
                  to_date:str=None
                ) -> ReportJob:
    api_utils.logRequest(req)

    try:
        drj =DeviceReportJob(tenant_id = tenant_id,
                            when=when, 
                            month_of_year=month_of_year,
                            from_date=from_date, 
                            to_date=to_date)

        drj_thread = DeviceReport(report_job=drj)
        drj_thread.start()
        return drj

    except Exception as ex:
        log.error(f"Could not create device report job: {ex}.")
        raise HTTPException(status_code=500, detail=f"Internal error.")




